if [ $# = 0 ]; then
    echo "Hampath: Wrong usage. Try hampath -help"
    exit
fi

hampathExecName=`basename $0`

#From where hampath is called
currentDir=`pwd`

hfun=./hfun.f90
efun=./efun.f90
parfun=./parfun.f90
output=`pwd`

option=""
helping=false
helpfun=""
listfun=""
newsimple=false
newmultip=false
helpgen=false
for i in $* ; do
    case "$option" in 
        -help)  helpfun=$i
                helpgen=false
                option="";;
        "")
            case "$i" in
            -help)  helping=true
                    helpgen=true  #il peut ne pas y avoir d'options après -help : dans ce cas la, un help general
                    option=$i;;
            -newSimpleShooting) newsimple=true;;
            -newMultipleShooting) newmultip=true;;
            -nss) newsimple=true;;
            -nms) newmultip=true;;
            -version)  sh ${hampathHome}/install/bin/version.sh ${hampathHome}
                        exit;;
            -*) echo "Invalid option! Try: hampath -help" 
                exit;;
            *.f90)      listfun=`echo $listfun " " $i`;;
            *)  echo "Invalid command! Try: hampath -help" 
                exit   ;;
            esac
    esac
done

pathhelp=$hampathHome/install/help
if [ $helping = "true" ]; then
    if [ $helpgen = "true" ]; then
        #Help general
        version=`cat $hampathHome/install/version.txt | sed "s/\([0-9]\)/\1./g"`
        less $pathhelp/general_help.txt | sed "s/VersionNumber/$version/"
    else
        case "$helpfun" in 
        hfun)       less $pathhelp/hfun_help.txt        ;;
        hamfun)     less $pathhelp/hfun_help.txt        ;;
        hvfun)      less $pathhelp/hvfun_help.txt       ;;
        dhvfun)     less $pathhelp/dhvfun_help.txt       ;;
        exphvfun)   less $pathhelp/exphvfun_help.txt    ;;
        expdhvfun)  less $pathhelp/expdhvfun_help.txt   ;;
        sfun)       less $pathhelp/sfun_help.txt        ;;
        sjac)       less $pathhelp/sjac_help.txt        ;;
        ssolve)     less $pathhelp/ssolve_help.txt      ;;
        hampath)    less $pathhelp/hampath_help.txt     ;;
        options)    less $pathhelp/options_help.txt     ;;
        hybrd)      less $pathhelp/hybrd._help.txt      ;;
        *) echo "No help for "$helpfun". Try hfun (or hamfun), hvfun, dhvfun, exphvfun, expdhvfun, sfun, sjac, ssolve, hampath, options, hybrd.";;
        esac
    fi
    exit
fi

if [ $newsimple = "true" ]; then
    echo "Copying all simple shooting example files: hfun.f90, efun.f90 and main.f90"
    echo " - hfun function is the true Hamiltonian."
    echo " - efun function represents the transversality and boundary conditions."
    echo " - main script solve the problem."
    cp $hampathHome/src/ex_simple_shooting/fortran/* $currentDir/
    echo "Done! You can now compile main.f90 (1) and execute main (2). Type:"
    echo "(1) ${hampathExecName} main.f90"
    echo "(2) main"
    exit
fi

if [ $newmultip = "true" ]; then
    echo "Copying all multiple shooting example files: hfun.f90, efun.f90 and main.f90"
    echo " - hfun function is the true Hamiltonian."
    echo " - efun function represents the transversality, boundary and intermediate conditions."
    echo " - main script solve the problem."
    cp $hampathHome/src/ex_multiple_shooting/fortran/* $currentDir/
    echo "Done! You can now compile (1) and execute main (2). Type:"
    echo "(1) ${hampathExecName} main.f90"
    echo "(2) main"
    exit
fi

echo "----------------------------------------------------------------------------------------------------"
echo " ${hampathExecName} execution..."
echo "----------------------------------------------------------------------------------------------------"

sh ${hampathHome}/install/bin/version.sh ${hampathHome}

echo ""
echo " Detection of the problem definition:"
testefun=true
testparf=true
if [ ! -f $hfun ]; then
    echo "    Error: hfun.f90 file not found. ${hampathExecName} will exit!"
    exit
fi
if [ ! -f $efun ]; then
    testefun=false
    echo "    Remark: efun.f90 file not found. ${hampathExecName} will take it into account."
fi
if [ ! -f $parfun ]; then
    testparf=false     #On copiera le parfun.f90 par defaut
    echo "    Remark: parfun.f90 file not found. ${hampathExecName} will use default parfun."
fi

echo ""
echo "----------------------------------------------------------------------------------------------------"
echo ""

#tmp=.hampath-tmp`date "+%Y%m%d%H%M%S"`
tmp=hampath-tmp`date "+%Y%m%d%H%M%S"`
echo " Temporary directory "$tmp" created for main processing..."
echo " Copy of all needed files in temporary directory"
mkdir $tmp
cp $hfun $tmp/hfun.f90
if [ $testefun = "true" ]; then
    cp $efun $tmp/efun.f90
else
    cp $hampathHome/src/utils/problemDef/efun.f90 $tmp/efun.f90
fi
if [ $testparf = "true" ]; then
    cp $parfun $tmp/parfun.f90
else
    cp $hampathHome/src/utils/problemDef/parfun.f90 $tmp/parfun.f90
fi
nomlistfunfile=listfun.txt
nommainProg=
listfunObjet=""
for i in $listfun ; do
    cp $i $tmp/$i
    fun=`basename $i '.f90'`
    listfunObjet=`echo $listfunObjet $fun".o"`
    nommainProg=$fun
done
echo $listfunObjet > $tmp/$nomlistfunfile
cd $tmp

#copie de la lib, compilation, lien et executable
cp $hampathHome/lib/fortran/* .
echo " Main process executing!"
echo ""
echo "----------------------------------------------------------------------------------------------------"
echo ""
if make -f $hampathHome/install/lib/makefile --silent hampathHome=${hampathHome} testefun=$testefun nomlistfunfile=$nomlistfunfile nommainProg=$nommainProg hampathInstallType=${hampathInstallType} diffAutoCmd
then
    if make -f $hampathHome/install/lib/makefile --silent hampathHome=${hampathHome} testefun=$testefun nomlistfunfile=$nomlistfunfile nommainProg=$nommainProg hampathInstallType=${hampathInstallType} problemCMD
    then
        cp ${nommainProg} ${currentDir}
        echo ""
        echo " All is complete! "$nommainProg" file ready for execution."
        echo ""
    else
        echo ""
        echo " Error during compilation!"
        echo ""
    fi
else
    echo ""
    echo " Error during Automatic differentiation!"
    echo ""
fi
echo "----------------------------------------------------------------------------------------------------"
echo ""
cd ${currentDir}
rm -rf $tmp
echo " Directory: "$tmp" removed."
echo ""
echo "----------------------------------------------------------------------------------------------------"
echo ""

#---------------------------------------------------------------------------------------------------------------------------------------------------------#
#---------------------------------------------------------------------------------------------------------------------------------------------------------#
# Written on 2009-2014
# by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis
