   module defs
    use ModgestListe
        implicit none

!!******************************************************************************************!!
    !Path of zeros during continuation : save or not
    !
!>  \ingroup globalVariables
!!  \brief File name to save homotopic path during execution
!!  \date   2009-2013
    character(32), save                                :: filenamepathg     = "nodef"
!>  \ingroup globalVariables
!!  \brief If 1, the path of zeros is saved in a file during continuation
!!  \date   2009-2013
    integer                                    ,  save :: doSavePathInFileg = -1

!!******************************************************************************************!!
    !Scaling dealing
!>  \ingroup globalVariables
!!  \brief If true, then scaling is performed before all integration
!!  \date   2009-2013
    logical                                    ,  save :: doScalingg        = .true.
!>  \ingroup globalVariables
!!  \brief Scaling is performed between 0 and kscalg
!!  \date   2009-2013
    integer                                    ,  save :: kscalg            = 1
!>  \ingroup globalVariables
!!  \brief Scaling is performed if the value is greater than scaltolg
!!  \date   2009-2013
    double precision                           ,  save :: scaltolg          = -1d0

!!******************************************************************************************!!
    !Stop after a turning point or not during continuation
    !
!>  \ingroup globalVariables
!!  \brief If true, then hampath() stops after having encountered a turning point
!!  \date   2009-2013
    logical                                    ,  save :: stopAtTurningPtg  = .false.

!!******************************************************************************************!!
    !Some parameters for continuation process
    !
!>  \ingroup globalVariables
!!  \brief Current tangent vector
!!  \date   2009-2013
    double precision, dimension(:), allocatable,  save :: tocg
!>  \ingroup globalVariables
!!  \brief Current dot product between two tagent vectors following
!!  \date   2009-2013
    double precision                           ,  save :: pdsg              = -1d0
!>  \ingroup globalVariables
!!  \brief Sign of the det for the right direction
!!  \date   2009-2013
    double precision                           ,  save :: sdetg             = -1d0
!>  \ingroup globalVariables
!!  \brief Sign of the det if bifurcation research
!!  \date   2009-2013
    double precision                           ,  save :: sdetbifg          = -1d0
!>  \ingroup globalVariables
!!  \brief Current value of the det of the augmented matrix
!!  \date   2009-2013
    double precision                           ,  save :: deterg            = -1d0
!>  \ingroup globalVariables
!!  \brief Current norm of the shootng function in hampath
!!  \date   2009-2013
    double precision                           ,  save :: normsg            = -1d0
!>  \ingroup globalVariables
!!  \brief Previous tangent vector in hampath
!!  \date   2009-2013
    double precision, dimension(:), allocatable,  save :: tocoldg
!>  \ingroup globalVariables
!!  \brief Previous tangent vector in bifurcation research
!!  \date   2009-2013
    double precision, dimension(:), allocatable,  save :: tocoldbifg
!>  \ingroup globalVariables
!!  \brief Used by hampath to know if dopri5 is initializating
!!  \date   2009-2013
    integer                                    ,  save :: hinitg            = -1
!>  \ingroup globalVariables
!!  \brief Initial parameters: before homotopy
!!  \date   2009-2013
    double precision, dimension(:), allocatable,  save :: par0g
!>  \ingroup globalVariables
!!  \brief Final parameters wanted after homotopy
!!  \date   2009-2013
    double precision, dimension(:), allocatable,  save :: parfg
!>  \ingroup globalVariables
!!  \brief Dimension of the parameters
!!  \date   2009-2013
    integer                                    ,  save :: lparg             = -1
!>  \ingroup globalVariables
!!  \brief Dimension of homotopic parameters
!!  \date   2009-2013
    integer                                    ,  save :: nparhomg          = -1
!>  \ingroup globalVariables
!!  \brief Indices of homotopic parameters
!!  \date   2009-2013
    integer,          dimension(:), allocatable,  save :: iparhomg

    !      double precision                           ,  save :: t0g     = -1d0
    !      double precision                           ,  save :: tfg     = -1d0
    !      integer                                    ,  save :: ntfg    = -1
    !      integer                                    ,  save :: neg     = -1

!!******************************************************************************************!!
    !display parameters
    !
!>  \ingroup globalVariables
!!  \brief If equals n then print lambda at each mod(n) steps
!!  \date   2009-2013
    integer, save          :: dispiterg  = -1
!>  \ingroup globalVariables
!!  \brief If equals 1 then print results
!!  \date   2009-2013
    integer, save          :: displayg   = -1

!!******************************************************************************************!!
    !Id to classify from which method, the integrator is called
    !
!>  \ingroup globalVariables
!!  \brief Id of integration in sfun()
!!  \date   2009-2013
    integer, parameter :: idSFung = 0
!>  \ingroup globalVariables
!!  \brief Id of integration in sjac()
!!  \date   2009-2013
    integer, parameter :: idSJacg = 1
!>  \ingroup globalVariables
!!  \brief Id of integration in exphvfun()
!!  \date   2009-2013
    integer, parameter :: idExpg  = 2
!>  \ingroup globalVariables
!!  \brief Id of integration in expdhvfun()
!!  \date   2009-2013
    integer, parameter :: idExpdg = 3
!>  \ingroup globalVariables
!!  \brief Id of integration in hampath()
!!  \date   2009-2013
    integer, parameter :: idHamg  = 4

!!******************************************************************************************!!
    !Integrator name for each method which use it
    !
!>  \ingroup globalVariables
!!  \brief Integrator choice for sfun()
!!  \date   2009-2013
    character(32), save    :: integsfung      = "nodef"
!>  \ingroup globalVariables
!!  \brief Integrator choice for sjac()
!!  \date   2009-2013
    character(32), save    :: integeqvarg     = "nodef"
!>  \ingroup globalVariables
!!  \brief Integrator choice for hampath()
!!  \date   2009-2013
    character(32), save    :: integhamg       = "nodef"
!>  \ingroup globalVariables
!!  \brief Integrator choice for exphvfun()
!!  \date   2009-2013
    character(32), save    :: integexphvfung  = "nodef"
!>  \ingroup globalVariables
!!  \brief Integrator choice for expdhvfun()
!!  \date   2009-2013
    character(32), save    :: integexpdhvfung = "nodef"

!!******************************************************************************************!!
    !derivative methods
    !
!>  \ingroup globalVariables
!!  \brief Derivative method choice
!!  \date   2009-2013
    character(32), save    :: derivativeg     = "nodef"

!!******************************************************************************************!!
    !tspan for exphvfun or expdhvfun
    !
!>  \ingroup globalVariables
!!  \brief The integration grid
!!  \date   2009-2013
    type(TLISTE), save     :: grilleg

!!******************************************************************************************!!
    !Parameters for Powell solver (hybrd/hybrj)
    !
!>  \ingroup globalVariables
!!  \brief maxfev for ssolve()
!!  \date   2009-2013
    integer, parameter     :: maxfevg = 2000
!>  \ingroup globalVariables
!!  \brief xtol for ssolve()
!!  \date   2009-2013
    double precision, save :: xtolg   = -1d0

!!******************************************************************************************!!
    !parameters for integration
    !
!>  \ingroup globalVariables
!!  \brief maxsteps of ODE integrator
!!  \date   2009-2013
    integer,          save    :: maxstepsg = -1
!>  \ingroup globalVariables
!!  \brief Absolute tolerance of ODE integrator
!!  \date   2009-2013
    double precision, save    :: atolg     = -1d0
!>  \ingroup globalVariables
!!  \brief Relative tolerance of ODE integrator
!!  \date   2009-2013
    double precision, save    :: rtolg     = -1d0
!>  \ingroup globalVariables
!!  \brief Maximum step size
!!  \date   2009-2013
    double precision, save    :: hmaxg     = -1d0

!!******************************************************************************************!!
    !parameters for continuation integration
    !
!>  \ingroup globalVariables
!!  \brief maxsteps of ODE integrator for hampath() method
!!  \date   2009-2013
    integer,          save    :: maxstepscontg  = -1
!>  \ingroup globalVariables
!!  \brief Absolute tolerance of ODE integrator for hampath() method
!!  \date   2009-2013
    double precision, save    :: atolcontg      = -1d0
!>  \ingroup globalVariables
!!  \brief Relative tolerance of ODE integrator for hampath() method
!!  \date   2009-2013
    double precision, save    :: rtolcontg      = -1d0
!>  \ingroup globalVariables
!!  \brief sfun maximal norm for hampath()
!!  \date   2009-2013
    double precision, save    :: NormeSfunMaxg  = -1d0
!>  \ingroup globalVariables
!!  \brief Maximum step size for hampath()
!!  \date   2009-2013
    double precision, save    :: hmaxcontg      = -1d0

!!******************************************************************************************!!
    !Parameters for dense output
    !
!>  \ingroup globalVariables
!!  \brief dense output tolerance to compute last homotopic parameter
!!  \date   2009-2013
    double precision, save    :: densetolg      = -1d0
!>  \ingroup globalVariables
!!  \brief relative tolerance used to check if the homotopic parameter still evolve
!!  \date   2009-2013
    double precision, save    :: densetolrelg   = -1d0

   end module defs

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

