
    !>     \ingroup commonIntegration
    !!     \brief Interface for dense output function adapted to the integrator.
    !!     \param[in] nameinte  integrator name
    !!     \param[in] i         index
    !!     \param[in] t         time at approximation
    !!     \param[out] y        approximate value
    !!     \param[in] told      last time
    !!     \param[in] hstep     steplength of current iteration: 0 .le. t-told .le. hstep
    !!     \param[in] lipar     dimension of ipar
    !!     \param[in] ipar      integer parameters
    !!     \param[in] ldpar     dimension of dpar
    !!     \param[in] dpar      double precision parameters
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine denseOutput(nameinte,i,t,y,told,hstep,lipar,ipar,ldpar,dpar)
        use utils
        implicit none
        character(32),      intent(in)                         :: nameinte
        integer,            intent(in)                         :: lipar,ldpar,i
        integer,            intent(in),    dimension(lipar)    :: ipar
        double precision,   intent(in)                         :: t,told,hstep
        double precision,   intent(out)                        :: y
        double precision,   intent(in),    dimension(ldpar)    :: dpar

        !local variables
        double precision :: CONTD5

        select case (nameinte)
            case ('dopri5')
                y = CONTD5(i,t,told,hstep,dpar,ipar,lipar)
            case default
                CALL printandstop('  ||| ERROR: Integrate -> Unknown integrator...')
        end select

    end subroutine denseOutput

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
