!!********************************************************************
!!
!!     Subroutine initTtfNe
!!
!>     \ingroup commonInitialization
!!     \brief  Initialize ntf (1 if tf free, 0 else) and ne (number of shooting equations) parameters
!!     \param[in]    fft     free final time or not
!!     \param[in]    m       number of arcs
!!     \param[in]    n       state dimension
!!     \param[out]   ntf     if the final time is free, ie fft is true, then ntf is 1
!!     \param[out]   ne      ne = 2*n*m+m-1+ntf is the dimension of the shooting function
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
        subroutine initTtfNe(fft,m,n,ntf,ne)
        use utils
        implicit none
        character(32),    intent(in)  :: fft
        integer,          intent(in)  :: m,n
        integer,          intent(out) :: ne,ntf

        select case (fft)
            case ('true')
                ntf = 1
            case ('false')
                ntf = 0
            case default
                CALL printandstop('invalid options.FreeFinalTime: true or false ?')
        end select

        ne  = 2*n*m+m-1+ntf

    end subroutine initTtfNe

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

