!!********************************************************************
!!
!!     Subroutine oneStepDopri5
!!
!>     \ingroup commonIntegration
!!     \brief Performs only one step of dopri5 integrator
!!     \param[in] fun
!!     \param[in] ny
!!     \param[in] t0
!!     \param[in] y0
!!     \param[in] h
!!     \param[in] lpar
!!     \param[in] par
!!     \param[out] t
!!     \param[out] y
!!     \param[out] ysti
!!     \param[out] value
!!     \param[in,out] k1
!!     \param[in,out] k2
!!     \param[in,out] k3
!!     \param[in,out] k4
!!     \param[in,out] k5
!!     \param[in,out] k6
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
subroutine oneStepDopri5(fun,ny,t0,y0,h,lpar,par,t,y,ysti,value,k1,k2,k3,k4,k5,k6)
      implicit none
    integer,                       intent(in)  :: ny
    double precision,                  intent(in)  :: t0,h
    double precision, dimension(ny),   intent(in)  :: y0
    double precision,                  intent(out) :: t
    double precision, dimension(ny),   intent(out) :: y,ysti
    double precision, dimension(ny),   intent(out) :: value
    integer,                       intent(in)  :: lpar
    double precision,dimension(lpar),  intent(in)  :: par
    double precision, dimension(ny),  intent(inout)  :: k1,k2,k3,k4,k5,k6

    !hvfun(nz,t,z,value,paraux,lparaux)
    interface
       Subroutine fun(neq,time,y,value,funpar,lfunpar)
         implicit none
         integer,      intent(in)                       :: neq
         integer,      intent(in)                       :: lfunpar
         double precision, intent(in)                       :: time
           double precision, intent(in),  dimension(lfunpar):: funpar
         double precision, intent(in),  dimension(neq)      :: y
         double precision, intent(out), dimension(neq)      :: value
       end Subroutine fun
    end interface

  !local variables
  double precision :: a21,a31,a32,a41,a42,a43,a51,a52,a53,a54,C2,C3,C4,C5
  double precision :: a61,a62,a63,a64,a65,a71,a73,a74,a75,a76
  double precision :: e1,e3,e4,e5,e6,e7,d1,d3,d4,d5,d6,d7 
  integer :: n

  CALL CDOPRI(c2,c3,c4,c5,e1,e3,e4,e5,e6,e7,                    &
                     a21,a31,a32,a41,a42,a43,a51,a52,a53,a54,   &
                     a61,a62,a63,a64,a65,a71,a73,a74,a75,a76,   &
                     d1,d3,d4,d5,d6,d7)

  if(h.eq.0d0) then

    t = t0
    y = y0
      call fun(ny,t,y,value,par,lpar)

  else

      n = ny/2

      call fun(ny,t0,y0,k1,par,lpar)

      t = t0 + C2*h
      y = y0 + h*a21*k1
      call fun(ny,t,y,k2,par,lpar)

      t = t0 + C3*h
      y = y0 + h*(a31*k1 + a32*k2)
      call fun(ny,t,y,k3,par,lpar)

      t = t0 + C4*h
      y = y0 + h*(a41*k1 + a42*k2 + a43*k3)
      call fun(ny,t,y,k4,par,lpar)

      t = t0 + C5*h
      y = y0 + h*(a51*k1 + a52*k2 + a53*k3 + a54*k4)
      call fun(ny,t,y,k5,par,lpar)

      t = t0 + h
      ysti = y0 + h*(a61*k1 + a62*k2 + a63*k3 + a64*k4 + a65*k5)
      call fun(ny,t,ysti,k6,par,lpar)

      y = y0 + h*(a71*k1 + a73*k3 + a74*k4 + a75*k5 + a76*k6)
      call fun(ny,t,y,value,par,lpar)

      k2 = value

  end if

end subroutine oneStepDopri5

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
