
!!********************************************************************
!!
!!     Subroutine expdhvrhs
!!
!>     @ingroup expdhvfunPackage
!!     @brief   RHS of expdhvfun
!!     @param[in]   neq     dimension of y
!!     @param[in]   t       time
!!     @param[in]   yscaled expdhvfun variable put in an array
!!     @param[in]   lparaux Number of optional parameters
!!     @param[in]   paraux  Optional parameters
!!
!!     @param[out]  value   derivative of y at t (RHS of expdhvfun)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
      Subroutine expdhvrhs(neq,t,yscaled,value,paraux,lparaux)
        use utils
        use derivatives
        integer,            intent(in)                      :: neq
        integer,            intent(in)                      :: lparaux
        double precision,   intent(in)                      :: t
        double precision,   intent(in),  dimension(lparaux) :: paraux
        double precision,   intent(in),  dimension(neq)     :: yscaled
        double precision,   intent(out), dimension(neq)     :: value

        !local variables
        integer                                 :: n,nbarc
        integer                                 :: k,iarc
        integer                                 :: ny
        integer                                 :: lpar
        double precision, dimension(neq)        :: hvz,scaletab,y,vscaled
        integer                                 :: vswitch

        lpar      = lparaux - 4 - neq

        iarc      = nint(paraux(lpar+1))
        nbarc     = nint(paraux(lpar+2))
        n         = nint(paraux(lpar+3))
        ny        = 2*n
        k         = nint(paraux(lpar+4))
        scaletab  = paraux(lpar+4+1:lpar+4+neq)

        !unscale
        call scalediv(neq,yscaled,scaletab,y)

        !value = hv(z(t)) for z(t)
        call hvfun(ny,t,y(1:ny),hvz(1:ny),paraux(1:lpar+2),lpar+2)
        vscaled(1:ny) = hvz(1:ny)

        !Compute value(...) = A * Y(...) for dz/dz0
        call dhvdz(t,n,y(1:ny),k,y(ny+1:neq),nbarc,iarc,lpar,paraux(1:lpar),vscaled(ny+1:neq))

        !scale rhs
        call scalemul(neq,vscaled,scaletab,value)

   end subroutine expdhvrhs

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
