!!********************************************************************
!!
!!     Subroutine getstatesdhv
!!
!>     @ingroup expdhvfunPackage
!!     @brief  Transforms the list etatsdhvG into an array for matlab dealing. Then remove the lists etatsdhvG and grilleG.
!!        @param[in]    n        State dimension
!!        @param[in]    k        Number of jacobi fields
!!        @param[in]    dimeT    Dimension of Ts (number of steps from expdhvfun)
!!
!!        @param[out]    Ts     Times of integration step from expdhvfun
!!        @param[out]    zs     Flow at each integration step
!!        @param[out]    dzs    Jacobi fields at each integration step
!!                  step from expdhvfun
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine getstatesdhv(n,k,dimeT,Ts,zs,dzs)
    use ModGestListe
    use adds
    use defs
    implicit none
    integer,                                    intent(in)  :: n
    integer,                                    intent(in)  :: k
    integer,                                    intent(in)  :: dimeT
    double precision, dimension(dimeT),         intent(out) :: Ts
    double precision, dimension(2*n,dimeT),     intent(out) :: zs
    double precision, dimension(2*n,dimeT*k),   intent(out) :: dzs

    !local variables
    integer                                 :: i
    double precision,dimension(2*n)         :: z
    double precision,dimension(2*n,k)       :: dz
    double precision                        :: time
    double precision, dimension(2*n*(k+1))  :: val

    do i=1,dimeT
      call EXTRAIRE(time,2*n*(k+1),val,etatsdhvG)
      Ts(dimeT-i+1)                         = time
      z                                     = val(1:2*n)
      dz                                    = reshape(val(2*n+1:2*n*(k+1)),(/2*n,k/))
      dzs(:,k*(dimeT-i)+1:k*(dimeT-i+1))    = dz
      zs(:,dimeT-i+1)                       = z
    end do

    call DETRUIRE(etatsdhvG)
    call DETRUIRE(grilleG)

   end subroutine getstatesdhv

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

