!!********************************************************************
!!
!!     Subroutine odeexpdhv
!!
!>     @ingroup expdhvfunPackage
!!     @brief   integration interface for expdhvfun
!!     \param[in]       fun         RHS
!!     \param[in]       neq         dimension of the state
!!     \param[in]       y0          initial point
!!     \param[in]       funpar      parameters given to fun during integration
!!     \param[in]       lfunpar     dimension of funpar
!!     \param[in]       ncpas       number of components of y for which we make a step control
!!     \param[in,out]   tin         initial time
!!     \param[in,out]   tout        final time
!!     \param[out]      yf          final point
!!     \param[out]      flag        information on the integration process, should be 1
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine odeexpdhv(fun,neq,y0,funpar,lfunpar,ncpas,tin,tout,yf,flag)
        use defs
        implicit none
        integer,            intent(in)                      :: lfunpar
        integer,            intent(in)                      :: neq
        integer,            intent(in)                      :: ncpas
        double precision,   intent(in),  dimension(neq)     :: y0
        double precision,   intent(in),  dimension(lfunpar) :: funpar
        double precision,   intent(inout)                   :: tin
        double precision,   intent(inout)                   :: tout
        double precision,   intent(out), dimension(neq)     :: yf
        integer,            intent(out)                     :: flag

        external soloutExpdhv

        interface
        Subroutine fun(neq,time,y,value,funpar,lfunpar)
            implicit none
            integer,            intent(in)                      :: neq
            integer,            intent(in)                      :: lfunpar
            double precision,   intent(in)                      :: time
            double precision,   intent(in),  dimension(lfunpar) :: funpar
            double precision,   intent(in),  dimension(neq)     :: y
            double precision,   intent(out), dimension(neq)     :: value
        end Subroutine fun
        end interface

        call integration(fun,neq,y0,funpar,lfunpar,ncpas,tin,tout,yf,flag,idExpdG,soloutExpdhv)

    end subroutine odeexpdhv

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
