!!********************************************************************
!!
!!     Subroutine soloutExpdhv
!!
!>     @ingroup expdhvfunPackage
!!     @brief   implementation of solout
!!     @param[in]   Niter       Number of iterations
!!     @param[in]   told        Previous time
!!     @param[in]   neq         Dimension of y
!!     @param[in]   dpar        Double precision parameters
!!     @param[in]   ldpar       Dimension of dpar
!!     @param[in]   ipar        Integer parameters
!!     @param[in]   lipar       Dimension of ipar
!!     @param[in]   fun         RHS
!!     @param[in]   lfunpar     Dimension of funpar
!!     @param[in,out]   time    Current time
!!     @param[in,out]   y       Current point, can be changed by dense output
!!     @param[in,out]   funpar  Parameters given to fun
!!     @param[in,out]   irtrn   Flag
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine soloutExpdhv(Niter,told,time,y,neq,dpar,ldpar,ipar,lipar,fun,funpar,lfunpar,irtrn)
        use defs
        use adds
        implicit none
        integer,            intent(in)                         :: Niter
        double precision,   intent(in)                         :: told
        double precision,   intent(inout)                      :: time
        double precision,   intent(inout), dimension(neq)      :: y
        integer,            intent(in)                         :: neq
        double precision,   intent(in),    dimension(ldpar)    :: dpar
        integer,            intent(in)                         :: ldpar
        integer,            intent(in),    dimension(lipar)    :: ipar
        integer,            intent(in)                         :: lipar
        double precision,   intent(inout), dimension(lfunpar)  :: funpar
        integer,            intent(in)                         :: lfunpar
        integer,            intent(inout)                      :: irtrn

        interface
            Subroutine fun(neq,time,y,value,funpar,lfunpar)
                implicit none
                integer,          intent(in)                        :: neq
                integer,          intent(in)                        :: lfunpar
                double precision, intent(in)                        :: time
                double precision, intent(in),  dimension(lfunpar)   :: funpar
                double precision, intent(in),  dimension(neq)       :: y
                double precision, intent(out), dimension(neq)       :: value
            end Subroutine fun
        end interface

        !local variables
        integer                          :: iarc,i,nbarc,lpar,n,k
        integer                          :: fini
        double precision, dimension(1)   :: val
        double precision, dimension(neq) :: scaletab,yg
        double precision                 :: t,hstep
        character(32)                    :: nameinte

        nameinte    = integexpdhvfunG
        scaletab    = 1d0

!---------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------
        !get n,lpar,par,nbar,iarc
        lpar        = lfunpar - 4 - neq
        k           = nint(funpar(lpar+4)) 
        n           = nint(funpar(lpar+3))
        nbarc       = nint(funpar(lpar+2)) 
        iarc        = nint(funpar(lpar+1)) 
        scaletab    = funpar(lpar+4+1:lpar+4+neq)

!---------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------
        if(GET_PARAM(grilleG).EQ.-1)then !if tspan given by the user is [t0 tf]
            call addstatesdhv(neq,scaletab,time,y)
        else                            !if tspan given by the user is [t0 t1 .. tf]
        !we have to find all ti from tspan (ie grilleG) contained in [told time]
        !and use dense output to get the value
        t     = GET_TIME(grilleG)
        fini  = 0
        hstep = time - told
            do while(((told-t)*(time-t).LE.0).AND.(fini.EQ.0))
                call EXTRAIRE(t,1,val,grilleG)
                if((time-t).NE.0)then
                    do i=1,neq
                        call denseOutput(nameinte,i,t,yg(i),told,hstep,lipar,ipar,ldpar,dpar)
                    end do
                else
                    yg=y
                end if
                call addstatesdhv(neq,scaletab,t,yg)
                if(LONGUEUR(grilleG).EQ.0)then
                    fini=1
                else
                    t = GET_TIME(grilleG)
                end if
            end do
        end if

  end subroutine soloutExpdhv

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

