!!********************************************************************
!!
!!     Subroutine exphvrhs
!!
!>     @ingroup exphvfunPackage
!!     @brief   RHS of exphvfun and sfun.
!!       @param[in]    t       time
!!       @param[in]    nz      State and costate dimension
!!       @param[in]    zscaled       State and adjoint state at t
!!       @param[in]    lparaux    Number of optional parameters
!!       @param[in]    paraux    Optional parameters
!!
!!       @param[out]   value   The RHS
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine exphvrhs(nz,t,zscaled,value,paraux,lparaux)
        use utils
        implicit none
        integer,      intent(in)                      :: nz
        integer,      intent(in)                      :: lparaux
        double precision, intent(in)                      :: t
        double precision, intent(in),  dimension(nz)      :: zscaled
        double precision, intent(in),  dimension(lparaux) :: paraux
        double precision, intent(out), dimension(nz)      :: value

        !local variables
        double precision,  dimension(nz) :: z, scaletab, hv
        integer                          :: lpar

        lpar     = lparaux - 2 - nz
        scaletab = paraux(lpar+2+1:lparaux)
        call scalediv(nz,zscaled,scaletab,z)
        call hvfun(nz,t,z,hv,paraux(1:lpar+2),lpar+2)
        call scalemul(nz,hv,scaletab,value)

    end subroutine exphvrhs

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
