!!********************************************************************
!!
!!     Subroutine getstates
!!
!>     @ingroup exphvfunPackage
!!     @brief  Transforms the list etatsG into an array and remove the list, for matlab dealing.
!!        @param[in]    n         State dimension
!!        @param[in]    dimeT     Dimension of Ts (number of steps from exphvfun)
!!
!!        @param[out]    Ts       Times of integration step from exphvfun
!!        @param[out]    Ys       Flow at each integration step from exphvfun
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine getstates(n,dimeT,Ts,Ys)
    use ModGestListe
    use adds
    use defs
    implicit none
    integer,                                intent(in)  :: n
    integer,                                intent(in)  :: dimeT
    double precision,dimension(dimeT),     intent(out)  :: Ts
    double precision,dimension(2*n,dimeT), intent(out)  :: Ys

    !local variables
    integer                             :: i
    double precision                    :: time
    double precision, dimension(2*n)    :: val

    do i=1,dimeT
      call EXTRAIRE(time,2*n,val,etatsG)
      Ts(dimeT-i+1) = time
      Ys(:,dimeT-i+1) = val
    end do

    call DETRUIRE(etatsG)
    call DETRUIRE(grilleG)

   end subroutine getstates

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

