module bifurcations
    use defs
    use adds
    use ModGestListe
    implicit none

    contains

!!********************************************************************
!!
!!     Subroutine findbif
!!
!>     @ingroup hampathPackage
!!     @brief   find bifurcations from the list pathG
!!        @param[in]    n1        dimension of point in path
!!        @param[in]    lnpath    Length of the path
!!        @param[in]    lparaux      Number of optional parameters
!!        @param[in]    paraux      Optional parameters
!!
!!        @param[out]   dimebif   Number of bifurcations found
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     Subroutine findbif(n1,dimebif,lnpath,lparaux,paraux)
        implicit none
        integer, intent(in)                      :: n1
        integer, intent(in)                      :: lnpath
        integer, intent(in)                      :: lparaux
        double precision, intent(in), dimension(lparaux) :: paraux
        integer, intent(out)                     :: dimebif

        !local variables
        integer                                 :: dime
        integer                                 :: k
        integer                                 :: ldo
        double precision                            :: deter
        double precision                            :: determinant
        double precision                            :: time
        double precision                            :: timebif
        double precision                            :: timeold
        double precision                            :: det
        double precision                            :: detold
        integer                            :: flag,dowrite
        double precision, dimension(2*n1+3)  :: val
        double precision, dimension(n1)  :: toc
        double precision, dimension(n1)  :: c
        double precision, dimension(n1)  :: cold
        double precision, dimension(n1)  :: cbif
        double precision, dimension(n1)  :: tocbif1
        double precision, dimension(n1)  :: tocbif2
        double precision, dimension(2*n1)  :: tocbif
        type(TNOEUD), pointer                    :: P

        dowrite = 0
        dimebif = 0
        if(lnpath>0) then

            dime = 2*n1+3

            determinant = 0
            ldo = lnpath

            call GET_NOEUD_SUIVANT(P,pathG)
            do k=1,ldo

                time = GET_TIME(P)
                val  = GET_VALEUR(P,dime)
                call GET_NOEUD_SUIVANT(P)

                c     = val(1:n1)
                toc   = val(1+n1:2*n1)
                deter = sign(1d0,val(dime-2))
                det   = val(dime-2)

                if(k==1) then
                    determinant = deter
                else
                    if (deter.ne.determinant) then
                        tocoldbifG = toc
                        call getBif(n1,deter,time,timeold,c,timebif,cbif,tocbif1,tocbif2,lparaux,paraux,flag)
                        if(flag.eq.0d0)then
                          tocbif(1:n1)      = tocbif1
                          tocbif(1+n1:2*n1) = tocbif2
                          call addbiftoc(2*n1,timebif,tocbif)
                          call addbifc(n1,timebif,cbif)
                        else
                          dowrite = 1
                        end if
                        determinant = deter
                    end if
                end if
                detold = det
                cold = c
                timeold = time
            end do

            dimebif = LONGUEUR(bifcG)

            if(dowrite.eq.1)then
               write(*,*) 'You must change the hampath tolerance to follow the path correctly.'
            end if

        end if

      end subroutine findbif

!!********************************************************************
!!
!!     Subroutine getBif
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine getBif(n1,detold,timeold,time,cold,timebif,cbif,tocbif1,tocbif2,lpar,par,flag)
        implicit none
        integer,                     intent(in)   :: n1
        double precision,                intent(in)  :: timeold,time,detold
        double precision, dimension(n1), intent(in)  :: cold
        double precision, intent(out)                 :: timebif
        double precision, dimension(n1), intent(out) :: cbif
        double precision, dimension(n1), intent(out) :: tocbif1
        double precision, dimension(n1), intent(out) :: tocbif2
        integer, intent(in)  :: lpar
        double precision, intent(in), dimension(lpar) :: par
        integer, intent(out) :: flag

        !local variables
        double precision, dimension(n1-1)  :: e
        double precision, dimension(n1)    :: tau1
        double precision, dimension(n1)    :: tau2
        double precision, dimension(2,2)   :: Hess
        double precision, dimension(n1)    :: tcbif
        double precision, dimension(n1,n1-1) :: ATbif

        call getU(n1,detold,timeold,time,cold,timebif,cbif,tcbif,ATbif,lpar,par)
        call getKer(n1,tcbif,ATbif,e,tau1,tau2)
        call getBifEqu(n1,e,tau1,tau2,cbif,Hess,lpar,par)
        if((Hess(1,1) * Hess(2,2) - Hess(1,2) * Hess(2,1)) .lt. 0d0) then
            flag = 0
        else
            flag = 1
        end if
        if(flag.eq.0d0)then
            call getVect(n1,Hess,tau1,tau2,tocbif1,tocbif2)
        else
            cbif    = 0d0
            tocbif1 = 0d0
            tocbif2 = 0d0
        end if

    end subroutine getBif

!!********************************************************************
!!
!!     Subroutine getU
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine getU(n1,detold,timeold,time,cold,timebif,cbif,tcbif,ATbif,lpar,par)
       implicit none
       integer,                     intent(in)  :: n1
       double precision, dimension(n1), intent(in)  :: cold
       double precision,                intent(in)  :: detold,timeold,time
       double precision,                intent(out)  :: timebif
       double precision, dimension(n1), intent(out) :: cbif
       double precision, dimension(n1), intent(out) :: tcbif
       double precision, dimension(n1,n1-1), intent(out) :: ATbif
       integer,                     intent(in)  :: lpar
       double precision,dimension(lpar),intent(in) :: par

       !local variables
       double precision  :: h1,h2,d1,d2,dm,eps,lstep
       double precision, dimension(n1)      :: cdummy
       double precision, dimension(n1)      :: tcdummy
       double precision, dimension(n1,n1-1) :: ATdummy
       integer :: ii

       eps = 1d0
       eps = epsilon(eps)
       eps = sign(abs(eps)**(1./3.),abs(eps))

       h1 = 0d0
       h2 = time-timeold

       lstep = abs(h2)

       ii   = 0

       do while ((lstep .gt. eps) .and. (ii.lt.15))

          ii = ii + 1

          call oneStep(n1,h1            ,cold,detold,d1,cdummy,tcdummy,ATdummy,lpar,par)
          call oneStep(n1,h2            ,cold,detold,d2,cbif  ,tcbif  ,ATbif,lpar,par)
          call oneStep(n1,h1+(h2-h1)/2d0,cold,detold,dm,cbif  ,tcbif  ,ATbif  ,lpar,par)

          timebif = timeold+h1+(h2-h1)/2d0
          lstep   = lstep / 2d0

          if(lstep .gt. eps) then
            if(d1*dm .lt. 0d0) then
              h2 = h1+(h2-h1)/2d0
            else
              h1 = h1+(h2-h1)/2d0
            end if
          end if

       end do

     end subroutine getU

!!********************************************************************
!!
!!     Subroutine oneStep
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine oneStep(n1,h,c0,det0,det,c,toc,AT,lpar,par)
      implicit none
      integer,                     intent(in)  :: n1
      double precision,                intent(in)  :: h
      double precision, dimension(n1), intent(in)  :: c0
      double precision,                intent(in)  :: det0
      double precision,                intent(out) :: det
      double precision, dimension(n1), intent(out) :: c
      double precision, dimension(n1), intent(out) :: toc
      double precision, dimension(n1,n1-1), intent(out) :: AT
      integer,                     intent(in)  :: lpar
      double precision,dimension(lpar),intent(in)  :: par

      !local variables
      double precision, dimension(n1) :: k1,k2,k3,k4,k5,k6
      double precision :: a21,a31,a32,a41,a42,a43,a51,a52,a53,a54
      double precision :: a61,a62,a63,a64,a65,a71,a73,a74,a75,a76

      a21=0.2D0
      a31=3.D0/40.D0
      a32=9.D0/40.D0
      a41=44.D0/45.D0
      a42=-56.D0/15.D0
      a43=32.D0/9.D0
      a51=19372.D0/6561.D0
      a52=-25360.D0/2187.D0
      a53=64448.D0/6561.D0
      a54=-212.D0/729.D0
      a61=9017.D0/3168.D0
      a62=-355.D0/33.D0
      a63=46732.D0/5247.D0
      a64=49.D0/176.D0
      a65=-5103.D0/18656.D0
      a71=35.D0/384.D0
      a73=500.D0/1113.D0
      a74=125.D0/192.D0
      a75=-2187.D0/6784.D0
      a76=11.D0/84.D0

      sdetbifG = sign(1d0,det0)

      if(h.eq.0d0) then

      call fcn(n1,c0,toc,lpar,par,det,AT)

      else

      call fcn(n1,c0,k1,lpar,par,det,AT)
      c = c0 + h*a21*k1

      call fcn(n1,c,k2,lpar,par,det,AT)
      c = c0 + h*(a31*k1 + a32*k2)

      call fcn(n1,c,k3,lpar,par,det,AT)
      c = c0 + h*(a41*k1 + a42*k2 + a43*k3)

      call fcn(n1,c,k4,lpar,par,det,AT)
      c = c0 + h*(a51*k1 + a52*k2 + a53*k3 + a54*k4)

      call fcn(n1,c,k5,lpar,par,det,AT)
      c = c0 + h*(a61*k1 + a62*k2 + a63*k3 + a64*k4 + a65*k5)

      call fcn(n1,c,k6,lpar,par,det,AT)
      c = c0 + h*(a71*k1 + a73*k3 + a74*k4 + a75*k5 + a76*k6)

      call fcn(n1,c,toc,lpar,par,det,AT)

      end if

     end subroutine oneStep

!!********************************************************************
!!
!!     Subroutine fcn
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine fcn(n1,c,toc,lparaux,paraux,deter,ATout)
       implicit none
       integer,      intent(in)                   :: n1
       integer,      intent(in)                   :: lparaux
       double precision, intent(in),  dimension(n1)   :: c
       double precision, intent(in),  dimension(lparaux) :: paraux
       double precision, intent(out), dimension(n1)   :: toc
       double precision,                intent(out) :: deter
       double precision, dimension(n1,n1-1), intent(out) :: ATout

       !local variables
       integer                           :: INFO
       integer                           :: n,nbarc,ne,ntf
       double precision, dimension(:), allocatable       :: ti
       double precision, dimension(:,:), allocatable      :: zi
       double precision, dimension(:,:), allocatable      :: expzi
       integer                           :: i,lpar
       double precision                      :: lambda,t0,tf
       double precision                      :: detR
       double precision                      :: detQ
       double precision                      :: pds
       double precision                      :: DDOT
       double precision, dimension(lparG)    :: par
       double precision, dimension(n1-1,n1)  :: A
       double precision, dimension(n1,n1-1)  :: AT
       double precision, dimension(n1,1)     :: e
       double precision, dimension(n1-1)     :: TAU
       double precision, dimension(n1-1)     :: WORK
       double precision, dimension(1)        :: WORK2

        nbarc    = nint(paraux(1))
        n        = nint(paraux(2))
        ne       = nint(paraux(3))
        ntf      = nint(paraux(4))
        t0       =      paraux(5)
        tf       =      paraux(6)
       lpar  = lparG

       allocate(ti(nbarc+1),zi(2*n,nbarc),expzi(2*n,nbarc))

       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)

       !AT = the transpose of the jacobian of S(y,lambda)
       call sjac(nbarc,ti,n,zi,lpar,par,expzi,1,lambda,ne,ntf,A)
       AT = transpose(A)
       ATout = AT

        toc = 0

        !computation of AT=QR
        call DGEQRF( n1, n1-1, AT, n1, TAU, WORK, n1-1, INFO )

        !computation of toc
        e = 0d0
        e(n1,1) = 1d0
        call DORMQR( 'L', 'N', n1, 1, n1-1, AT, n1, TAU, e, n1, WORK2, 1, INFO )
        toc = e(:,1)

        !computation of det(Q) and det(R) where AT=QR
        detQ = 0d0
        detR = 1d0
        do i=1,n1-1
            if(TAU(i)>0d0) then
                detQ = detQ + 1
            end if
            detR = detR * AT(i,i)
        end do
        detQ = -1**detQ
        deter = detQ*detR

        !check if we scan the curve in the right direction
        if(sdetbifG.ne.sign(1d0,deter)) then
            toc   = -1d0*toc
            deter = -1d0*deter
        end if
        pds = DDOT(n1,toc,1,tocoldbifG,1)
        if(pds.lt.0d0) then
            pds   = -1d0*pds
            toc   = -1d0*toc
            sdetbifG  = -1d0*sdetbifG
            deter = -1d0*deter
        end if

        deallocate(ti,zi,expzi)

     end subroutine fcn

!!********************************************************************
!!
!!     Subroutine getKer
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine getKer(n1,tcbif,ATbif,e,tau1,tau2)
       implicit none
       integer,                       intent(in)  :: n1
       double precision, dimension(n1),   intent(in)  :: tcbif
       double precision, dimension(n1,n1-1), intent(in) :: ATbif
       double precision, dimension(n1-1), intent(out) :: e
       double precision, dimension(n1),   intent(out) :: tau1
       double precision, dimension(n1),   intent(out) :: tau2

      !local variables
       integer                           :: INFO
       double precision, dimension(n1,n1)    :: B
       double precision, dimension(n1-1,n1)  :: A
       double precision, dimension(n1)       :: S
       double precision, dimension(n1-1,n1-1):: U
       double precision, dimension(n1,n1)    :: VT
       double precision, dimension(5*n1)     :: WORKSVD

       tau1 = tcbif

       A = transpose(ATbif)
       B(1:n1-1,:) = A
       B(n1,:) = tau1

       call DGESVD( 'N', 'A', n1, n1, B, n1, S, U, 1, VT, n1, WORKSVD, 5*n1, INFO )
       tau2 = VT(n1,:)

       call DGESVD( 'A', 'N', n1-1, n1, A, n1-1, S, U, n1-1, VT, 1, WORKSVD, 5*(n1-1), INFO )
       e = U(:,n1-1)

     end subroutine getKer

!!********************************************************************
!!
!!     Subroutine getBifEqu
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine getBifEqu(n1,e,tau1,tau2,cbif,Hess,lparaux,paraux)
       implicit none
       integer,                       intent(in)  :: n1
        double precision, dimension(n1-1), intent(in)  :: e
        double precision, dimension(n1),   intent(in)  :: tau1
        double precision, dimension(n1),   intent(in)  :: tau2
        double precision, dimension(n1),   intent(in)  :: cbif
        double precision, dimension(2,2),  intent(out) :: Hess
         integer,                       intent(in)  :: lparaux
        double precision, dimension(lparaux), intent(in)  :: paraux

       !local variables
        double precision, dimension(lparG)   :: par
       double precision                      :: eps,t0,tf
       double precision                      :: lambda
       integer                           :: n,nbarc,ne,ntf
       double precision, dimension(:), allocatable        :: ti
       double precision, dimension(:,:), allocatable      :: zi,expzi
       integer                           :: lpar
       integer                           :: flag
        double precision, dimension(n1-1)     :: s
        double precision, dimension(n1)       :: c
       double precision                      :: gE0
       double precision                      :: g00
       double precision                      :: gME0
       double precision                      :: g0E
       double precision                      :: g0ME
       double precision                      :: gEE
       double precision                      :: gMEME
       double precision                      :: gEME
       double precision                      :: gMEE
        double precision                      :: DDOT

       lpar = lparG
        nbarc    = nint(paraux(1))
        n        = nint(paraux(2))
        ne       = nint(paraux(3))
        ntf      = nint(paraux(4))
        t0       =      paraux(5)
        tf       =      paraux(6)

       allocate(ti(nbarc+1),zi(2*n,nbarc),expzi(2*n,nbarc))

       eps = 1d0
       eps = epsilon(eps)
       eps = sign(abs(eps)**(1./3.),abs(eps))

       !gE0
       c = cbif + eps * tau1 + 0d0 * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gE0 = DDOT(n1-1,e,1,s,1)

       !g00
       c = cbif + 0d0 * tau1 + 0d0 * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       g00 = DDOT(n1-1,e,1,s,1)

       !gME0
       c = cbif - eps * tau1 + 0d0 * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gME0 = DDOT(n1-1,e,1,s,1)

       !g0E
       c = cbif + 0d0 * tau1 + eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       g0E = DDOT(n1-1,e,1,s,1)

       !g0ME
       c = cbif + 0d0 * tau1 - eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       g0ME = DDOT(n1-1,e,1,s,1)

       !gEE
       c = cbif + eps * tau1 + eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gEE = DDOT(n1-1,e,1,s,1)

       !gMEME
       c = cbif - eps * tau1 - eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gMEME = DDOT(n1-1,e,1,s,1)

       !gEME
       c = cbif + eps * tau1 - eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gEME = DDOT(n1-1,e,1,s,1)

       !gMEE
       c = cbif - eps * tau1 + eps * tau2
       call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
       call getPar(lambda,lpar,par)
       call sfun(nbarc,ti,n,zi,lpar,par,expzi,s,flag)
       gMEE = DDOT(n1-1,e,1,s,1)

       Hess(1,1) = (gE0 - 2*g00 + gME0)/eps**2
       Hess(1,2) = 0.25*(gEE + gMEME - gEME - gMEE)/eps**2
       Hess(2,1) = Hess(1,2)
       Hess(2,2) = (g0E - 2*g00 + g0ME)/eps**2

       deallocate(ti,zi,expzi)

     end subroutine getBifEqu

!!********************************************************************
!!
!!     Subroutine getVect
!!
!>     @ingroup hampathPackage
!!     @brief
!!     @param[in]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     subroutine getVect(n1,Hess,tau1,tau2,tocbif1,tocbif2)
       implicit none
       integer,                       intent(in)  :: n1
        double precision, dimension(2,2),  intent(in)  :: Hess
        double precision, dimension(n1),   intent(in)  :: tau1
        double precision, dimension(n1),   intent(in)  :: tau2
        double precision, dimension(n1),   intent(out) :: tocbif1
        double precision, dimension(n1),   intent(out) :: tocbif2

        !local variables
        double precision                 :: l1
        double precision                 :: l2
        double precision                 :: trH
        double precision                 :: detH
        double precision                 :: delta
        double precision, dimension(2)   :: Y1
        double precision, dimension(2)   :: Y2
        double precision, dimension(2)   :: X1
        double precision, dimension(2)   :: X2
        double precision, dimension(2)   :: P1
        double precision, dimension(2)   :: P2
        double precision                      :: DDOT

        trH = Hess(1,1) + Hess(2,2)
        detH = Hess(1,1) * Hess(2,2) - Hess(1,2) * Hess(2,1)
        delta = trH**2 - 4*detH

        l1 = 0.5*(trH + sqrt(delta))
        l2 = 0.5*(trH - sqrt(delta))

        Y1(2)=1
        Y1(1)=sqrt(-1*l2/l1)
        Y2(2)=1
        Y2(1)=-1*sqrt(-1*l2/l1)

        P1(2)=1
        P1(1)=-1*Hess(1,2)/(Hess(1,1)-l1)
        P2(2)=1
        P2(1)=-1*Hess(1,2)/(Hess(1,1)-l2)
        P1 = P1 / sqrt(P1(1)**2+P1(2)**2)
        P2 = P2 / sqrt(P2(1)**2+P2(2)**2)

        X1(1)=P1(1)*Y1(1)+P2(1)*Y1(2)
        X1(2)=P1(2)*Y1(1)+P2(2)*Y1(2)
        X2(1)=P1(1)*Y2(1)+P2(1)*Y2(2)
        X2(2)=P1(2)*Y2(1)+P2(2)*Y2(2)

        tocbif1 = X1(1)*tau1 + X1(2)*tau2
        tocbif1 = tocbif1 / sqrt(ddot(n1,tocbif1,1,tocbif1,1))
        tocbif2 = X2(1)*tau1 + X2(2)*tau2
        tocbif2 = tocbif2 / sqrt(ddot(n1,tocbif2,1,tocbif2,1))

      end subroutine getVect


end module bifurcations

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

