!!********************************************************************
!!
!!     Subroutine getbif
!!
!>     @ingroup hampathPackage
!!     @brief  Transform the lists bifcG and biftocG into arrays for matlab dealing.
!!        @param[in]    n           State dimension
!!        \param[in]    ne          Dimension of the shooting function
!!        @param[in]    dimebif     Number of bifurcations detected by hampath
!!
!!        @param[out]    bifctab    all (z,lambda,s) at each bifurcation
!!        @param[out]    biftoctab  the two tangent vectors for each bifurcation
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine getbif(n,ne,dimebif,bifctab,biftoctab)
      use ModGestListe
      use defs
      use adds
      implicit none
      integer, intent(in)                                    :: n,ne
      integer, intent(in)                                    :: dimebif
      double precision,dimension(ne+2,dimebif)    , intent(out) :: bifctab
      double precision,dimension(2*(ne+1),dimebif), intent(out) :: biftoctab

      !local variables
      integer                            :: i
      integer                            :: ny
      double precision                       :: time
      double precision, dimension(ne+1)     :: valc
      double precision, dimension(2*(ne+1)) :: valtoc

      ny = 2*n
      !we extract information for each bifurcation
      do i=1,dimebif
         call EXTRAIRE(time,ne+1,valc,bifcG)
         bifctab(1:ne+1,i)  = valc !the bifurcation point
         bifctab(ne+1+1,i)  = time !the time (arc length)
         call EXTRAIRE(time,2*(ne+1),valtoc,biftocG) !the two tangent vectors
         biftoctab(:,i)     = valtoc
      enddo

      !we remove the lists
      call DETRUIRE(bifcG)
      call DETRUIRE(biftocG)

    end subroutine getbif

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
