!!********************************************************************
!!
!!     Subroutine savePathToFile
!!
!>     @ingroup hampathPackage
!!     @brief Save current continuation step in a file
!!       \param[in] nbarc   Number of arcs
!!       \param[in] n       State dimension
!!       \param[in] ne      Dimension of the shooting function
!!       \param[in] ntf     If 1, then the final time is free and tf is an independent variable
!!       \param[in] t0      initial time
!!       \param[in] tf      final time
!!       \param[in] lpar    Number of optional parameters
!!       \param[in] time    Current time integration
!!       \param[in] dime    Dimension of valpath
!!       \param[in] valpath Value to save in a file
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine savePathToFile(nbarc,n,ne,ntf,t0,tf,lpar,time,dime,valpath)
        use defs
        use utils
        integer,            intent(in) :: nbarc, n, ne, lpar, dime, ntf
        double precision,   intent(in) :: time, valpath(dime), t0, tf

        !local variables
        double precision, dimension(lpar)       :: par
        double precision, dimension(nbarc+1)    :: ti
        double precision, dimension(2*n,nbarc)  :: zi
        double precision, dimension(ne+1)       :: toci,c
        double precision                        :: s,det,norms,ps,lambda
        integer                             :: i,j,unitfile
        CHARACTER(len=(nbarc+1)*25+25+5)    :: LINE
        CHARACTER(len=25)                   :: valStr1, valstr2, blanks

        s       = time ! abscisse curviligne
        toci    = valpath(ne+1+1:2*(ne+1))
        det     = valpath(dime-2)
        norms   = valpath(dime-1)
        ps      = valpath(dime-0)
        c       = valpath(1:ne+1)
        call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
        call getPar(lambda,lpar,par)

        blanks="                         "

        unitfile=20

        !   [lout,tiout,ziout,sout,viout,bifs,dets,normS,ps,flag] = hampath(parspan,ti,zi,options)
        ! print zi
        open(unit=unitfile,file=filenamepathG,action="write",status="old",position="append")

        !lout
        WRITE (valStr1,'(a)') 'lout  = [lout  ['
        do i=1,lpar
            WRITE (valStr2,'(e23.15)') par(i)
            if(i.eq.1)then
                LINE = valStr1 // valStr2
            else
                LINE = blanks // valStr2
            end if
            if(i.eq.lpar)then
                LINE = LINE(1:25+25) // "]];"
            end if
            WRITE (unit=unitfile,fmt='(a)') LINE
        end do

        !ti
        WRITE (valStr1,'(a)') 'tiout = [tiout ['
        DO j=1,nbarc+1
            WRITE (valStr2,'(e23.15)') ti(j)
            if(j.eq.1)then
                LINE = valStr1 // valStr2
            elseif(j.eq.nbarc+1)then
                LINE = LINE(1:25+25*(j-1)) // valStr2 // ']];'
                WRITE (unit=unitfile,fmt='(a)') LINE
            else
                LINE = LINE(1:25+25*(j-1)) // valStr2
            end if
        END DO

        !zi
        WRITE (valStr1,'(a)') 'ziout = [ziout ['
        DO i = 1, 2*n
            if(nbarc.eq.1)then
                WRITE (valStr2,'(e23.15)') zi(i,1)
                if(i.eq.1)then
                    LINE = valStr1 // valStr2
                elseif(i.eq.2*n)then
                    LINE = blanks // valStr2 // "]];"
                else
                    LINE = blanks // valStr2
                end if
                WRITE (unit=unitfile,fmt='(a)') LINE
            else
            DO j=1,nbarc
                WRITE (valStr2,'(e23.15)') zi(i,1)
                if(j.eq.1)then
                    if(i.eq.1)then
                        LINE = valStr1 // valStr2
                    else
                        LINE = blanks // valStr2
                    end if
                elseif(j.eq.nbarc)then
                    if(i.eq.2*n)then
                        LINE = LINE(1:25+25*(j-1))//valStr2//']];'
                    else
                        LINE = LINE(1:25+25*(j-1)) // valStr2
                    end if
                    WRITE (unit=unitfile,fmt='(a)') LINE
                else
                    LINE = LINE(1:25+25*(j-1)) // valStr2
                end if
            END DO
        end if
        END DO

        !s
        WRITE (valStr1,'(a)') 'sout  = [sout  ['
        WRITE (valStr2,'(e23.15)') s
        LINE = valStr1 // valStr2 // ']];'
        WRITE (unit=unitfile,fmt='(a)') LINE

        !vi
        WRITE (valStr1,'(a)') 'viout = [viout ['
        do i=1,ne+1
            WRITE (valStr2,'(e23.15)') toci(i)
            if(i.eq.1)then
                LINE = valStr1 // valStr2
            elseif(i.eq.ne+1)then
                LINE = blanks // valStr2 // "]];"
            else
                LINE = blanks // valStr2
            end if
            WRITE (unit=unitfile,fmt='(a)') LINE
        end do

        !det
        WRITE (valStr1,'(a)') 'dets  = [dets  ['
        WRITE (valStr2,'(e23.15)') det
        LINE = valStr1 // valStr2 // ']];'
        WRITE (unit=unitfile,fmt='(a)') LINE

        !normS
        WRITE (valStr1,'(a)') 'normS = [normS ['
        WRITE (valStr2,'(e23.15)') norms
        LINE = valStr1 // valStr2 // ']];'
        WRITE (unit=unitfile,fmt='(a)') LINE

        !ps
        WRITE (valStr1,'(a)') 'ps    = [ps    ['
        WRITE (valStr2,'(e23.15)') ps
        LINE = valStr1 // valStr2 // ']];'
        WRITE (unit=unitfile,fmt='(a)') LINE

        close(unit=unitfile)

    end subroutine savePathToFile

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
