!!********************************************************************
!!
!!     Subroutine hvfun
!!
!>     @ingroup hvfunPackage
!!     @brief   Defines the second member of the BVP.
!!       @param[in]    t       time
!!       @param[in]    nz      State and costate dimension
!!       @param[in]    z       State and adjoint state at t
!!       @param[in]    lparaux    Number of optional parameters
!!       @param[in]    paraux    Optional parameters
!!
!!       @param[out]   value   The second member of the BVP
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
        Subroutine hvfun(nz,t,z,value,paraux,lparaux)
            implicit none
            integer,      intent(in)                      :: nz
            integer,      intent(in)                      :: lparaux
            double precision, intent(in)                      :: t
            double precision, intent(in),  dimension(nz)      :: z
            double precision, intent(in),  dimension(lparaux) :: paraux
            double precision, intent(out), dimension(nz)      :: value

            !Local variables
            double precision, dimension(lparaux-2)  :: l0
            double precision, dimension(nz)         :: work 
            double precision                        :: H
            integer                                 :: i
            integer                                 :: n,iarc,nbarc,lpar

            n        = nz/2
            work     = 0d0
            l0       = 0d0
            lpar     = lparaux - 2
            nbarc    = nint(paraux(lpar+2))
            iarc     = nint(paraux(lpar+1))
            DO i = 1, n
                work(i+n) = 1.0D0
                call HFUN_D(t, 0d0, n, z, work, nbarc, iarc, lpar, paraux(1:lpar), l0, H, value(i)) !dh/dp
                work(i+n) = 0.0D0

                work(i  ) =-1.0D0
                call HFUN_D(t, 0d0, n, z, work, nbarc, iarc, lpar, paraux(1:lpar), l0, H, value(i+n)) !-dh/dx
                work(i  ) = 0.0D0
            END DO

        end subroutine hvfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
