!!********************************************************************
!!
!!     Subroutine dexpzirhs
!!
!>     @ingroup sjacPackage
!!     @brief   RHS of variational equations
!!     @param[in]   neq     State dimension
!!     @param[in]   t        time
!!     @param[in]   yscaled     variable put in an array
!!     @param[in]   lparaux     Number of optional parameters
!!     @param[in]   paraux     Optional parameters
!!
!!     @param[out]  value    RHS of variational equations
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
        Subroutine dexpzirhs(neq,t,yscaled,value,paraux,lparaux)
            use utils
            use derivatives
            implicit none
            integer,          intent(in)                        :: neq
            integer,          intent(in)                        :: lparaux
            double precision, intent(in),  dimension(lparaux)   :: paraux
            double precision, intent(in)                        :: t
            double precision, intent(in),  dimension(neq)       :: yscaled
            double precision, intent(out), dimension(neq)       :: value

            !local variables
            integer                             :: ny,niarc,nl,lpar,iarc,nbarc
            double precision, dimension(neq)    :: hvz,scaletab,y,vscaled
            double precision, dimension(neq)    :: b !b(ny)
            double precision                    :: lambda

            lpar      = lparaux - 6 - neq
            iarc      = nint(paraux(lpar+1))
            nbarc     = nint(paraux(lpar+2))
            ny        = nint(paraux(lpar+3))
            niarc     = nint(paraux(lpar+4))
            nl        = nint(paraux(lpar+5))
            lambda    =      paraux(lpar+6)
            scaletab  = paraux(lpar+6+1:lpar+6+neq)

            !unscale
            call scalediv(neq,yscaled,scaletab,y)

            !value = hv(z(t)) for z(t)
            call hvfun(ny,t,y(1:ny),hvz(1:ny),paraux(1:lpar+2),lpar+2)
            vscaled(1:ny) = hvz(1:ny)

            !Compute value(...) = A * Y(...) for dz/dz0, dz/dt0, dz/dl
            call dhvdz(t,ny/2,y(1:ny),(neq-ny)/ny,y(ny+1:neq),nbarc,iarc,lpar,paraux(1:lpar),vscaled(ny+1:neq))

            if(nl.eq.1)then
                !value(...) = A * Y(...) + b for dz/dl
                !b = (dhv/dlambda)(t,y)
                call dhvdlambda(t,ny/2,y(1:ny),nbarc,iarc,lambda,lpar,paraux(1:lpar),b(1:ny))
                vscaled(neq-ny+1:neq) = vscaled(neq-ny+1:neq) + b(1:ny)
            end if

            !scale rhs
            call scalemul(neq,vscaled,scaletab,value)

        end subroutine dexpzirhs

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

