program main
    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !Steps of the main program:
    !1) include modules used
    !2) declare all variables
    !3) initialize options
    !4) Make a first shoot
    !5) Make a continuation on lambda parameter from 0 to 0.5
    !
    !Problem definition
    !min tf
    !
    !dot x = v
    !dot v = -lambda v**2 + u
    !
    !x(0) = -1, x(1) = 0
    !v(0) = -1, v(1) = 0
    !
    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !1) Module used in the main program
    !
!    use mod_expdhvfun
    use mod_exphvfun
!    use mod_hamfun
    use mod_hampath
!    use mod_hvfun
!    use mod_hybrd
    use mod_options
!    use mod_sfun
!    use mod_sjac
    use mod_ssolve

    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !2) variables declaration
    !
    implicit none
    type(TOptions)   :: options
    !ssolve variables
    double precision :: zi(4,2), ti(3), par(1), ssol(10), u
    integer          :: nfev,njev

    !hampath variables
    double precision,  dimension(1,2)              :: parspan
    double precision,  dimension(:,:), allocatable :: parout
    double precision,  dimension(:),   allocatable :: tiout
    double precision,  dimension(:,:), allocatable :: ziout
    double precision,  dimension(:),   allocatable :: sout
    double precision,  dimension(:,:), allocatable :: viout
    double precision,  dimension(:),   allocatable :: dets
    double precision,  dimension(:),   allocatable :: normS
    double precision,  dimension(:),   allocatable :: ps

    !exphvfun variables
    double precision, allocatable, dimension(:)     :: tout
    double precision, allocatable, dimension(:,:)   :: exphv

    !Common variables
    integer          :: flag

    !Print options
    !call getOption or setOption to deal with hampath options
    write(*,*) "----------------------------------------------------------------------------"
    call setOption(options,'FreeFinalTime','true')
    call printOptions(options)

    !First shooting: lambda = 0
    write(*,*) "----------------------------------------------------------------------------"
    write(*,*) "Shooting..."
    ti      = (/  0d0,  2d0, 4d0 /)
    zi(:,1) = (/ -1d0, -1d0, 0.1d0, 0.1d0 /)
    par     = (/ 0d0  /) !lambda
    call exphvfun(ti, zi(:,1), ti, options, par, tout, exphv, flag) !Get value at t1
    zi(:,2) = exphv(:,2)
    call ssolve(ti, zi, options, par, ti, zi, ssol, nfev, njev, flag)

    !Homotopy on lambda, from 0 to 0.5
    write(*,*) "----------------------------------------------------------------------------"
    write(*,*) "Homotopy..."   
    parspan(:,1) = par
    parspan(:,2) = par
    parspan(1,2) = 0.5d0 !for homotopy
    call hampath(parspan,ti,zi,options,parout,tiout,ziout,sout,viout,dets,normS,ps,flag)  

    deallocate(parout,tiout,ziout,sout,viout,dets,normS,ps,tout,exphv)

end program main

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
