clear;
close all;
format long;
startup;

%
%Problem definition
%min tf
%
%dot x = v
%dot v = -lambda v**2 + u
%
%x(0) = -1, x(1) = 0
%v(0) = -1, v(1) = 0
%

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 1: parameters initialization\n');

    ti0     = [0.0 2.0 4.0]            % t0, t1 and tf
    zi0     = [-1.0 -1.0 0.1 0.1]'     % x(0), v(0), p_x(0), p_v(0)
    par0    = 0.0                      % lambda_0
    parf    = 0.5                      % lambda_f : for homotopy on lambda
    options = hampathset('FreeFinalTime','true')
    [ tout, exphv, flag ] = exphvfun(ti0, zi0, ti0, options, par0);
    zi0     = [ zi0 exphv(:,2)];
    nbarc   = length(ti0) - 1;

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 2: first shooting\n');

    [tisol,zisol,ssol,nfev,njev,flag] = ssolve(ti0,zi0,options,par0);

zisol

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 3: continuation on lambda from %g to %g\n',par0,parf);

    parspan = [par0 parf];
    [ parout, tiout, ziout, sout, viout, dets, normS, ps, flag ] = hampath(parspan,tisol,zisol,options);

    lout = parout;

    tif  = tiout(end-nbarc:end);
    z0f  = ziout(:,end-nbarc+1);
    parf = parout(:,end);

    % Figures
    % Paths
    figure;
    subplot(2,5,1);plot(ziout(3,1:nbarc:end),lout,'k');xlabel('p_x(0)');ylabel('\lambda'); drawnow; title('Paths');
    subplot(2,5,6);plot(ziout(4,1:nbarc:end),lout,'k');xlabel('p_v(0)');ylabel('\lambda'); drawnow;

    % Solution
    [ tout, exphv, flag ] = exphvfun([tif(1) tif(end)], z0f, tif, options, parf);

    subplot(2,5,2);plot(tout,exphv(1,:),'b');xlabel('t');ylabel('x(t)');   drawnow; title('State solution');
    subplot(2,5,3);plot(tout,exphv(3,:),'b');xlabel('t');ylabel('p_x(t)'); drawnow; title('Co-state solution');
    subplot(2,5,7);plot(tout,exphv(2,:),'b');xlabel('t');ylabel('v(t)');   drawnow;
    subplot(2,5,8);plot(tout,exphv(4,:),'b');xlabel('t');ylabel('p_v(t)'); drawnow;

    % Control
    u = control(tout,exphv,tif,parf);
    subplot(2,5,[4 5]);plot(tout,u,'r');xlabel('t');ylabel('u(t)'); drawnow; title('Control');

