program main
    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !Steps of the main program:
    !1) include modules used
    !2) declare all variables
    !3) initialize options
    !4) Make a first shoot
    !5) Make a continuation on lambda parameter from 0 to 1
    !
    !Problem definition
    !min int_0^1 |u|**2
    !
    !dot x = v
    !dot v = -lambda v**2 + u
    !
    !x(0) = -1, x(1) = 0
    !v(0) = -1, v(1) = 0
    !
    !remark : all initial and final datas for x and v are put in a vector of parameters
    !
    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !1) Module used in the main program
    !
    use mod_options
    use mod_ssolve
    use mod_hampath

    !----------------------------------------------------------------------------------------------------!
    !----------------------------------------------------------------------------------------------------!
    !2) variables declaration
    !
    implicit none
    type(TOptions)   :: options
    !ssolve variables
    double precision :: z0(4), ti(2), par(5), ssol(4), u
    integer          :: nfev,njev

    !hampath variables
    double precision,  dimension(5,2)              :: parspan
    double precision,  dimension(:,:), allocatable :: parout
    double precision,  dimension(:),   allocatable :: tiout
    double precision,  dimension(:,:), allocatable :: ziout
    double precision,  dimension(:),   allocatable :: sout
    double precision,  dimension(:,:), allocatable :: viout
    double precision,  dimension(:),   allocatable :: dets
    double precision,  dimension(:),   allocatable :: normS
    double precision,  dimension(:),   allocatable :: ps

    !Common variables
    integer          :: flag

    !Print options
    !call getOption or setOption to deal with hampath options
    write(*,*) "----------------------------------------------------------------------------"
    call printOptions(options)

    !First shooting: lambda = 0
    write(*,*) "----------------------------------------------------------------------------"
    write(*,*) "Shooting..."

    ti  = (/  0d0,  1d0 /)
    z0  = (/ -1d0, -1d0, 0.1d0, 0.1d0       /)
    par = (/ -1d0, -1d0,   0d0,   0d0, 0d0  /) !x0, v0, xf, vf, lambda
    call ssolve(ti, z0, options, par, ti, z0, ssol, nfev, njev, flag)

    !Homotopy on lambda, from 0 to 1
    write(*,*) "----------------------------------------------------------------------------"
    write(*,*) "Homotopy..."   
    parspan(:,1) = par
    parspan(:,2) = par
    parspan(5,2) = 1d0 !for homotopy
    call hampath(parspan,ti,z0,options,parout,tiout,ziout,sout,viout,dets,normS,ps,flag)  

    deallocate(parout,tiout,ziout,sout,viout,dets,normS,ps)

end program main

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
