clear;
close all;
format long;
startup;

% Problem definition
% min int_0^1 u^2
% dot(x) = v
% dox(v) = -lambda v^2 + u
% x(0) = x_0, x(1) = x_f, v(0) = v_0, v(1) = v_f

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 1: parameters initialization\n');

    ti0     = [0.0 1.0]                % t0 and tf
    zi0     = [-1.0 -1.0 0.1 0.1]'     % x(0), v(0), p_x(0), p_v(0)
    par     = [-1.0 -1.0 0.0 0.0 0.0]';% x_0, v_0, x_f, v_f, lambda_0
    par0    = par                      % par_0
    parf    = par; parf(5) = 1.0       % par_f with lambda_f = 1.0 : for homotopy on lambda
    options = hampathset

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 2: first shooting\n');

    [tisol,zisol,ssol,nfev,njev,flag] = ssolve(ti0,zi0,options,par0);

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 3: continuation on lambda from %g to %g\n',par0(5),parf(5));

    parspan = [par0 parf];
    [ parout, tiout, ziout, sout, viout, dets, normS, ps, flag ] = hampath(parspan,tisol,zisol,options);

    lout = parout(5,:);

    z0f  = ziout(:,end);
    parf = parout(:,end);

    % Figures
    % Paths
    figure;
    subplot(2,5,1);plot(ziout(3,:),lout,'k');xlabel('p_x(0)');ylabel('\lambda'); drawnow; title('Paths');
    subplot(2,5,6);plot(ziout(4,:),lout,'k');xlabel('p_v(0)');ylabel('\lambda'); drawnow;

    % Solution
    [ tout, exphv, flag ] = exphvfun(tisol, z0f, options, parf);

    subplot(2,5,2);plot(tout,exphv(1,:),'b');xlabel('t');ylabel('x(t)');   drawnow; title('State solution');
    subplot(2,5,3);plot(tout,exphv(3,:),'b');xlabel('t');ylabel('p_x(t)'); drawnow; title('Co-state solution');
    subplot(2,5,7);plot(tout,exphv(2,:),'b');xlabel('t');ylabel('v(t)');   drawnow;
    subplot(2,5,8);plot(tout,exphv(4,:),'b');xlabel('t');ylabel('p_v(t)'); drawnow;

    % Control
    u = control(tout,exphv,parf);
    subplot(2,5,[4 5]);plot(tout,u,'r');xlabel('t');ylabel('u(t)'); drawnow; title('Control');

%-------------------------------------------------------------------------------------------------------------%
fprintf('\nStep 4: optimality checked on final solution\n');

   n = length(zi0)/2;
   k = n; % regular case, k = n : fixed tf
   dz0 = [ zeros(n) ; eye(n)];

   [ tout, z, dz, flag ] = expdhvfun(tisol, z0f, dz0, options, parf);

   sv=[]; de=[];
   for j = 1:length(tout)
     dx    = dz(1:n,1+(j-1)*k:j*k);
     sv(j) = min(svd(dx));
     de(j) = det(dx);
   end;

   subplot(2,5,[9 10]); plot(tout,sv,'m'); hold on; plot(tout,max(sv)*sign(de),'k--'); hold on; xlabel('t');
   legend('\sigma_m_i_n','sign(det(\delta x))','location','West'); drawnow; title('Conjugate points');


