function [fjac, flag] = sjac(ti,zi,options,par,indexpar)
%-------------------------------------------------------------------------------------------
%
%    sjac (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Computes the Jacobian of the shooting-homotopic function
%
%    Options used
%
%        Derivative, FreeFinalTime, MaxStepsOde, MaxStepSizeOde, ODE, TolOdeAbs, 
%        TolOdeRel, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [fjac, flag] = sjac(ti,zi,options,par)
%        [fjac, flag] = sjac(ti,zi,options,par,indexpar)
%
%    Inputs
%
%        ti      - real row vector,
%                    ti = [t0 tf]                    : simple shooting
%                    ti = [t0 t1 ... t_{nbarc-1} tf] : multiple shooting
%        zi      - real matrix, shooting variable,
%                    zi = [z(t0) ... z(t_{nbarc-1})]
%        options - struct, hampathset options
%        par     - real row vector, par in hfun and efun, par=[] if no parameters
%        indexpar- integer vector, index of parameters for which the derivative is computed
%
%    Outputs
%
%        fjac    - real matrix, jacobian of the homotopic function
%        flag    - integer, should be 1 (ode output).
%
%-------------------------------------------------------------------------------------------
if(nargout~=2)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end
if(nargin==4)
indexpar=[];
else
nmin=min(indexpar);
nmax=max(indexpar);
if(nmin<1)
error('indexpar must contain values between 1 and length(par)');
end
if(nmax>length(par))
error('indexpar must contain values between 1 and length(par)');
end
end

str=num2str(clock); str=strrep(str,' ',''); str=strrep(str,'.',''); 
strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'sjacIn' strFinal '.txt'];
while(exist(nomfilein,'file'))
    strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'sjacIn' strFinal '.txt'];
end;
nomfileout=[ 'sjacOu' strFinal '.m']; nomfun='sjac'; nominterface='matlab'; nomfunint=[ 'sjacOu' strFinal];

%!Description of input file
%!options
%!n
%!nbarc
%!lpar
%!lipar
%!ti(1,nbarc+1)
%!zi(2*n,nbarc)
%!par(lpar,1)
%!ipar(lipar,1)

%Create file in
[nn,ntz] = size(zi);        if(mod(nn,2)~=0)    error('Variable zi must have an even number of rows!');     end; n = nn/2;
[m,nti]  = size(ti);        if(m>1)             error('Variable ti must be a row vector!');                 end; nbarc = nti - 1;
                            if(ntz~=nti-1)      error('Variables ti must have one column more than zi!');   end;
[lpar,m] = size(par);       if(m>1)             error('Variable par must be a column vector!');             end;
[lipar,m]= size(indexpar);  if(m>1)             error('Variable indexpar must be a column vector!');        end;

fid = fopen(nomfilein,'w');
fprintf(fid,hampathget(options));
fprintf(fid,'%i\n%i\n%i\n%i\n',nbarc,n,lpar,lipar);
for i=1:lipar
    fprintf(fid,'%i\n',indexpar(i));
end;
fclose(fid);
save(nomfilein,'-ascii','-double','-append','ti','zi','par');

%Call main routine
p = mfilename('fullpath'); [pathstr,name,ext] = fileparts(p);
gfortranMatlab(1); flag = system([pathstr '/mainProg ' nomfilein ' ' nomfileout ' ' nomfun ' ' nominterface ' ' nomfunint]); gfortranMatlab(2);

%Get results from file out
try
    [fjac,flag] = eval(nomfunint);
catch err
    if(exist(nomfilein,'file'))  flagAux = system(['rm ' nomfilein]);  end;
    if(exist(nomfileout,'file')) flagAux = system(['rm ' nomfileout]); end;
    error('Problem during execution!');
end;

flagAux = system(['rm ' nomfilein]);
flagAux = system(['rm ' nomfileout]);

return;

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!


