function [hv,dhv] = dhvfun(arg1,arg2,arg3,arg4,arg5)
%-------------------------------------------------------------------------------------------
%
%    dhvfun (needs hfun.f90)
%
%    Description
%
%        Computes the second member of the variational system associated to H.
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [hv, dhv] = dhvfun(t, z, dz,     par) : single arc
%        [hv, dhv] = dhvfun(t, z, dz, ti, par) : multiple arcs
%
%    Inputs
%
%        t    -  real row vector, t(i) = i-th time
%        z    -  real matrix, z(:,i) = i-th state and costate
%        dz   -  real matrix, dz(:,(i-1)*k+1:i*k) = the k i-th Jacobi fields
%        ti   -  real row vector, in multiple shooting case, ti = [t0 t1 ... t_nbarc-1 tf]
%        par  -  real row vector, parameters given to hfun par=[] if no parameters
%
%    Outputs
%
%        hv   -  real matrix, hamiltonian vector field at time(s) t
%        dhv  -  real matrix, Linearized of the Hamiltonian vector field at time(s) t
%
%-------------------------------------------------------------------------------------------
if(nargout~=2)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end

t=arg1;
z=arg2;
dz=arg3;
if(nargin==nrhs0min)
ti =[t(1) t(end)];
par=arg4;
elseif(nargin==nrhs0min+1)
ti=arg4;
par=arg5;
end

[hv, dhv] = dhvfun_m(t, z, dz, ti, par);

%!!******************************************************************************************!!
%! Written on 2014                                                                           !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

