#include "fintrf.h"
C======================================================================
#if 0
C
C     dhvfun_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2014
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       [hv, dhv] = dhvfun_m(t, z, dz, ti, par);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C
C       .. Input Arguments ..
C
C       t           : double precision vector
C       z           : double precision matrix
C       dz          : double precision matrix
C       ti          : double precision vector
C       par         : double precision vector
C
C       .. Output Arguments
C
C       hv          : double precision matrix
C       dhv         : double precision matrix
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0, icur
      parameter(nlhs0 = 2)
      parameter(nrhs0 = 5)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      integer mxIsNumeric

C   Pointers to input/output arguments
      mwPointer tPr,zPr,tiPr,parPr,dzPr
      mwPointer hvPr,dhvPr

C   Array information
      mwPointer nt,mz,nz,nti,mpar,kdz,mdz,ndz

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input : t
C       Columns > 0, Rows = 1. isNumeric.
      icur = 1
      nt = mxGetN(prhs(icur))
      if(mxGetM(prhs(icur)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:mt',
     +                           't must have one row.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:tNonNumeric',
     +                           't must be a numeric array.')
      endif
      tPr = mxGetPr(prhs(icur))

C       .. Get input : z
C       Columns > 0, Rows > 0. isNumeric.
      icur = icur + 1
      mz = mxGetM(prhs(icur))
      nz = mxGetN(prhs(icur))
      if(mod(mz,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:mz',
     +                           'z must have even number of rows.')
      end if
      if(nz .ne. nt) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:nz',
     +              'z must have the same number of rows than t.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:zNonNumeric',
     +                           'z must be a numeric array.')
      endif
      zPr = mxGetPr(prhs(icur))

C       .. Get input : dz
C       Columns > 0, Rows > 0. isNumeric.
      icur = icur + 1
      mdz = mxGetM(prhs(icur))
      ndz = mxGetN(prhs(icur))
      if(mod(mdz,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:mdz',
     +                          'dz must have an even number of rows.')
      end if
      if(ndz .lt. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:dz',
     +                           'dz must have at least one column.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:dzNonNumeric',
     +                           'dz must be a numeric array.')
      endif
      if(mz.ne.mdz)then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:zdz',
     +              'z and dz must have the same number of rows.')
      end if
      kdz  = ndz/nt
      dzPr = mxGetPr(prhs(icur))

C       .. Get input : ti
C       Columns > 0, Rows = 1. isNumeric.
      icur = icur + 1
      nti = mxGetN(prhs(icur))
      if(mxGetM(prhs(icur)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:mti',
     +                           'ti must have one row.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(icur))

C       .. Get input : par
C       Columns = 1, Rows > 0. isNumeric.
      icur = icur + 1
      mpar = mxGetM(prhs(icur))
      if(mxGetN(prhs(icur)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:dhvfun:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:dhvfun:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(icur))

C-----------------------------------------------------------------------

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(mz, nt, 0)
      hvPr    = mxGetPr(plhs(1))

      plhs(2) = mxCreateDoubleMatrix(mz, nt*kdz, 0)
      dhvPr   = mxGetPr(plhs(2))

C     .. Computation ..
      call dhvfunmexint(tPr,mz,kdz,zPr,dzPr,nti,tiPr,mpar,
     +                  parPr,nt,hvPr,dhvPr)

      RETURN
      END

C-----------------------------------------------------------------------
C     Computational routine
      subroutine dhvfunmexint(tPr,mz,kdz,zPr,dzPr,nti,tiPr,mpar,
     +                        parPr,nt,hvPr,dhvPr)

      implicit none

      mwPointer tPr,zPr,dzPr,tiPr,parPr
      mwPointer hvPr,dhvPr
      mwPointer nt,mz,nz,nti,mpar,kdz

      double precision t(nt),z(mz,nt),ti(nti),par(mpar),hv(mz,nt)
      double precision dz(mz,nt*kdz), dhv(mz,nt*kdz)

      call mxCopyPtrToReal8(tPr,t,nt)
      call mxCopyPtrToReal8(zPr,z,mz*nt)
      call mxCopyPtrToReal8(dzPr,dz,mz*nt*kdz)
      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call dhvfunint(t,mz/2,kdz,z,dz,nti-1,ti,mpar,par,nt,hv,dhv)

      call mxCopyReal8ToPtr(hv,hvPr,mz*nt)
      call mxCopyReal8ToPtr(dhv,dhvPr,mz*nt*kdz)

      return
      end



