#include "fintrf.h"
C======================================================================
#if 0
C
C     MYFUNCTION_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       out = MYFUNCTION_m(t, z, ti, par);
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 1)
      parameter(nrhs0 = 4)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      integer mxIsNumeric

C   Pointers to input/output arguments
      mwPointer tPr,zPr,tiPr,parPr
      mwPointer hPr

C   Array information
      mwPointer nt,mz,nz,nti,mpar

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input 1 : t
C       Columns > 0, Rows = 1. isNumeric.
      nt = mxGetN(prhs(1))
      if(mxGetM(prhs(1)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:mt',
     +                           't must have one row.')
      end if
      if(mxIsNumeric(prhs(1)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:tNonNumeric',
     +                           't must be a numeric array.')
      endif
      tPr = mxGetPr(prhs(1))

C       .. Get input 2 : z
C       Columns > 0, Rows > 0. isNumeric.
      mz = mxGetM(prhs(2))
      nz = mxGetN(prhs(2))
      if(mod(mz,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:mz',
     +                           'z must have an even number of rows.')
      end if
      if(nz .ne. nt) then
          call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:nz',
     +              'z must have the same number of rows than t.')
      end if
      if(mxIsNumeric(prhs(2)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:zNonNumeric',
     +                           'z must be a numeric array.')
      endif
      zPr = mxGetPr(prhs(2))

C       .. Get input 3 : ti
C       Columns > 0, Rows = 1. isNumeric.
      nti = mxGetN(prhs(3))
      if(mxGetM(prhs(3)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:mti',
     +                           'ti must have one row.')
      end if
      if(nti .le. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:exphvfun:nti',
     +                           'Check that ti=[t0 t1 tm-1 tf].')
      end if
      if(mxIsNumeric(prhs(3)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(3))

C       .. Get input 4 : par
C       Columns = 1, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(4))
      if(mxGetN(prhs(4)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(4)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:MYFUNCTION:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(4))

C-----------------------------------------------------------------------

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(MYDIMENSION, nt, 0)
      hPr     = mxGetPr(plhs(1))

C     .. Computation ..
      call MYFUNCTIONmexint(tPr,mz,zPr,nti,tiPr,mpar,parPr,nt,hPr)

      RETURN
      END

C-----------------------------------------------------------------------
C     Computational routine
      subroutine MYFUNCTIONmexint(tPr,mz,zPr,nti,tiPr,mpar,parPr,nt,hPr)

      mwPointer tPr,zPr,tiPr,parPr
      mwPointer hPr
      mwPointer nt,mz,nz,nti,mpar

      double precision t(nt),z(mz,nt),ti(nti),par(mpar),
     +                  h(MYDIMENSION,nt)

      call mxCopyPtrToReal8(tPr,t,nt)
      call mxCopyPtrToReal8(zPr,z,mz*nt)
      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call MYFUNCTIONint(t,mz/2,z,nti-1,ti,mpar,par,nt,h)

      call mxCopyReal8ToPtr(h,hPr,MYDIMENSION*nt)

      return
      end


