function [tout,z,dz,flag] = expdhvfun(tspan,z0,dz0,varargin)
%-------------------------------------------------------------------------------------------
%
%    expdhvfun (needs hfun.f90)
%
%    Description
%
%        Computes the chronological exponential of the variational system associated to hv.
%
%    Options used
%
%        MaxStepsOde, MaxStepSizeOde, ODE, TolOdeAbs, TolOdeRel, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [tout,z,dz,flag] = expdhvfun(tspan, z0, dz0,     options, par) : single arc
%        [tout,z,dz,flag] = expdhvfun(tspan, z0, dz0, ti, options, par) : multiple arcs
%
%    Inputs
%
%        tspan   - real row vector, tspan = [tspan1 tspan2 ... tspanf] must be sorted and
%                    included in ti, if any.
%        z0      - real vector, initial flow
%        dz0     - real matrix, initial Jacobi fields
%        ti      - real row vector, in multiple shooting case ti = [t0 t1 ... t_nbarc-1 tf]
%                    must be increasing.
%        options - struct vector, hampathset options
%        par     - real row vector, parameters given to hfun par=[] if no parameters
%
%    Outputs
%
%        tout    - real row vector, time at each integration step.
%        z       - real matrix, flow at time tout
%        dz      - real matrix, Jacobi fields at time tout
%        flag    - integer, flag should be 1 (ODE integrator output).
%
%-------------------------------------------------------------------------------------------
if(nargout~=4)
error('wrong number of outputs: try help');
end

nrhs0min = 5;
if(nargin<nrhs0min)
    error('wrong syntax: try help');
end;

if(nargin==nrhs0min)
    ti =[tspan(1) tspan(end)];
    options=varargin{1};
    par=varargin{2};
elseif(nargin==nrhs0min+1)
    ti=varargin{1};
    options=varargin{2};
    par=varargin{3};
end

[tout,z,dz,flag]  = expdhvfun_m(tspan,z0,dz0,par,options.TolOdeAbs,options.TolOdeRel, ...
                                        options.ODE,options.MaxStepsOde,ti,options.MaxStepSizeOde, ...
                                        options.TolScaling);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

