#include "fintrf.h"
C======================================================================
#if 0
C
C     expdhvfun_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C     [ tout, expdhv, flag ] = expdhvfun_m(tspan,dz0,par,           ...
C                           options.TolOdeAbs,options.TolOdeRel,    ...
C                           options.ODE,options.MaxStepsOde,ti,     ...
C                           options.MaxStepSizeOde,options.TolScaling);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C                       Inputs
C
C       .. Input Arguments ..
C
C       tspan           : double precision vector
C       dz0             : double precision matrix
C       par             : double precision vector
C       TolOdeAbs       : double precision
C       TolOdeRel       : double precision
C       ODE             : char*32
C       MaxStepsOde     : integer
C       ti              : double precision vector
C       MaxStepSizeOde  : double precision
C       TolScaling      : double precision
C
C       .. Output Arguments
C
C       tout        : double precision vector
C       expdhv      : double precision matrix
C       flag        : integer
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 4)
      parameter(nrhs0 = 11)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      mwPointer mxCreateString, mxIsInt8, mxCreateNumericArray
      integer mxGetString, mxIsNumeric, mxIsChar, statu
      integer*4 mxClassIDFromClassName

C   Arguments for mxCreateNumericArray
      integer*4 classid, complexflag
      mwSize ndim, dims(2)

C   Pointers to input/output arguments
      mwPointer tspPr,z0Pr,dz0Pr,parPr,atPr,rtPr,odePr,msPr,tiPr
      mwPointer toPr,zPr,dzPr,flagPr,mssPr,tscPr

C   Real, char or integer associated to pointer argument
      double precision at,rt,msD,flagD,mss,tsc
      integer ms,flag,ipr
      character*32 ode

C   Array information
      mwPointer ntsp,mz0,nz0,mdz0,ndz0,mpar,nti
      integer nto
      mwSize maxBuf
      parameter(maxBuf = 32)
      mwPointer strlen

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

      ipr = 0

C       .. Get input: tspan
C       Columns > 0, Rows = 1. isNumeric.
      ipr = ipr + 1
      ntsp = mxGetN(prhs(ipr))
      if(mxGetM(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:tspan',
     +                           'tspan must have one row.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:tspanNonNumeric',
     +                           'tspan must be a numeric array.')
      endif
      tspPr = mxGetPr(prhs(ipr))

C       .. Get input: z0
C       Columns = 1, Rows > 1. isNumeric.
      ipr = ipr + 1
      mz0 = mxGetM(prhs(ipr))
      nz0 = mxGetN(prhs(ipr))
      if(mod(mz0,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:mz0',
     +                          'z0 must have an even number of rows.')
      end if
      if(nz0 .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:z0',
     +                           'z0 must have one column.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:dz0NonNumeric',
     +                           'z0 must be a numeric array.')
      endif
      z0Pr = mxGetPr(prhs(ipr))


C       .. Get input: dz0
C       Columns > 1, Rows > 0. isNumeric.
      ipr = ipr + 1
      mdz0 = mxGetM(prhs(ipr))
      ndz0 = mxGetN(prhs(ipr))
      if(mod(mdz0,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:mdz0',
     +                          'dz0 must have an even number of rows.')
      end if
      if(ndz0 .lt. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:dz0',
     +                           'dz0 must have at least one column.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:dz0NonNumeric',
     +                           'dz0 must be a numeric array.')
      endif
      dz0Pr = mxGetPr(prhs(ipr))

      if(mz0.ne.mdz0)then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:z0dz0',
     +              'z0 and dz0 must have the same number of rows.')
      end if

C       .. Get input: par
C       Columns = 1, Rows > 0. isNumeric.
      ipr = ipr + 1
      mpar = mxGetM(prhs(ipr))
      if(mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(ipr))


C       .. Get input: TolOdeAbs
C       Columns = 1, Rows = 1. isNumeric.
      ipr = ipr + 1
      if(mxGetM(prhs(ipr))*mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolOdeAbs',
     +                           'TolOdeAbs must be scalar.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolOdeAbsNonNumeric',
     +                           'TolOdeAbs must be a numeric scalar.')
      endif
      atPr = mxGetPr(prhs(ipr))
      call mxCopyPtrToReal8(atPr,at,1)

C       .. Get input: TolOdeRel
C       Columns = 1, Rows = 1. isNumeric.
      ipr = ipr + 1
      if(mxGetM(prhs(ipr))*mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolOdeRel',
     +                           'TolOdeRel must be scalar.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolOdeRelNonNumeric',
     +                           'TolOdeRel must be a numeric scalar.')
      endif
      rtPr = mxGetPr(prhs(ipr))
      call mxCopyPtrToReal8(rtPr,rt,1)

C       .. Get input: ODE
C       Columns = 0, Rows <= 32. isChar.
      ipr = ipr + 1
      if(mxIsChar(prhs(ipr)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:odeNonString',
     +                           'ODE must be a string.')
      endif
      if(mxGetM(prhs(ipr)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:odeNonRowVector',
     +                           'ODE must be a row vector.')
      endif
      strlen = mxGetM(prhs(ipr))*mxGetN(prhs(ipr))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:odeOverMaxBuf',
     +                           'ODE max length sticked out.')
      endif
      odePr = prhs(ipr)
      statu = mxGetString(odePr,ode,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:odeReadError',
     +                           'ODE error reading string.')
      endif

C       .. Get input: MaxStepsOde
C       Columns = 1, Rows = 1. isInt8.
      ipr = ipr + 1
      if(mxGetM(prhs(ipr))*mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:MaxStepsOde',
     +                            'MaxStepsOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:MaxStepsOdeNonInt',
     +                           'MaxStepsOde must be an integer.')
      endif
      msPr = mxGetPr(prhs(ipr))
      call mxCopyPtrToReal8(msPr,msD,1)
      ms = nint(msD)


C       .. Get input: ti
C       Columns > 0, Rows = 1. isNumeric.
      ipr = ipr + 1
      nti = mxGetN(prhs(ipr))
      if(mxGetM(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:mti',
     +                           'ti must have one row.')
      end if
      if(nti .le. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:nti',
     +                           'Check that ti=[t0 t1 tm-1 tf].')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(ipr))


C       .. Get input: MaxStepSizeOde
C       Columns = 1, Rows = 1. isNumeric.
      ipr = ipr + 1
      if(mxGetM(prhs(ipr))*mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:MaxStepSizeOde',
     +                           'MaxStepSizeOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
      call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:MaxStepSizeOdeNonNum',
     +                      'MaxStepSizeOde must be a numeric scalar.')
      endif
      mssPr = mxGetPr(prhs(ipr))
      call mxCopyPtrToReal8(mssPr,mss,1)

C       .. Get input: TolScaling
C       Columns = 1, Rows = 1. isNumeric.
      ipr = ipr + 1
      if(mxGetM(prhs(ipr))*mxGetN(prhs(ipr)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolScaling',
     +                           'TolScaling must be scalar.')
      end if
      if(mxIsNumeric(prhs(ipr)) .eq. 0) then
        call mexErrMsgIdAndTxt ('MATLAB:expdhvfun:TolScalingNonNumeric',
     +                           'TolScaling must be a numeric scalar.')
      endif
      tscPr = mxGetPr(prhs(ipr))
      call mxCopyPtrToReal8(tscPr,tsc,1)

C-----------------------------------------------------------------------

C    .. Create outputs ..
C    .. Create output flag ..
C      classid  = mxClassIDFromClassName('int8')
C      complexflag = 0
C      ndim     = 1
C      dims(1)  = 1
C      plhs(3) = mxCreateNumericArray(ndim,dims,classid,complexflag)
      plhs(4) = mxCreateDoubleMatrix(1, 1, 0)
      flagPr   = mxGetPr(plhs(4))

C     .. Computation ..
      call expdhvfunmexint(mdz0,ndz0,z0Pr,dz0Pr,mpar,parPr,ntsp,tspPr,
     +                          nti,tiPr,at,rt,ode,ms,flag,nto,mss,tsc)

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(1, nto, 0)
      toPr    = mxGetPr(plhs(1))

      plhs(2) = mxCreateDoubleMatrix(mdz0, nto, 0)
      zPr     = mxGetPr(plhs(2))

      plhs(3) = mxCreateDoubleMatrix(mdz0, nto*ndz0, 0)
      dzPr    = mxGetPr(plhs(3))

      call getstatesdhvmex(mdz0,ndz0,nto,toPr,zPr,dzPr)

      flagD = dble(flag)
      CALL mxCopyReal8ToPtr(flagD, flagPr, 1)

      RETURN
      END

C-----------------------------------------------------------------------
C     Computational routine
      subroutine expdhvfunmexint(mdz0,ndz0,z0Pr,dz0Pr,mpar,parPr,ntsp,
     +                  tspPr,nti,tiPr,at,rt,ode,ms,flag,nto,mss,tsc)

      implicit none

      mwPointer tspPr,z0Pr,dz0Pr,parPr,tiPr
      double precision at,rt,mss,tsc
      integer ms
      character*32 ode
      mwPointer ntsp,mdz0,ndz0,mpar,nti
      integer nto,flag

      double precision z0(mdz0),dz0(mdz0,ndz0),par(mpar)
      double precision ti(nti),tsp(ntsp)

      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(tspPr,tsp,ntsp)
      call mxCopyPtrToReal8(z0Pr,z0,mdz0)
      call mxCopyPtrToReal8(dz0Pr,dz0,mdz0*ndz0)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call expdhvfunint(mdz0/2,ndz0,z0,dz0,mpar,par,ntsp,tsp,nti-1,ti,
     +                  flag,nto,at,rt,ode,ms,mss,tsc)

      return
      end

      subroutine getstatesdhvmex(mdz0,ndz0,nto,toPr,zPr,dzPr)

      implicit none

      mwPointer toPr,zPr,dzPr,mdz0,ndz0
      integer nto

      double precision tout(nto),z(mdz0,nto),dz(mdz0,nto*ndz0)

      call getstatesdhv(mdz0/2,ndz0,nto,tout,z,dz)

      call mxCopyReal8ToPtr(tout,toPr,nto)
      call mxCopyReal8ToPtr(z,zPr,mdz0*nto)
      call mxCopyReal8ToPtr(dz,dzPr,mdz0*nto*ndz0)
      return
      end


