function [ tout, exphv, flag ]  = exphvfun(arg1,arg2,arg3,arg4,arg5)
%-------------------------------------------------------------------------------------------
%
%    exphvfun (needs hfun.f90)
%
%    Description
%
%        Computes the chronological exponential of the Hamiltonian vector field hv
%        defined by h.
%
%    Options used
%
%        MaxStepsOde, MaxStepSizeOde, ODE, TolOdeAbs, TolOdeRel, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [tout,exphv,flag ] = exphvfun(tspan, z0,     options, par) : single arc
%        [tout,exphv,flag ] = exphvfun(tspan, z0, ti, options, par) : multiple arcs
%
%    Inputs
%
%        tspan   - real row vector, tspan = [tspan1 tspan2 ... tspanf] must be sorted and
%                    included in ti, if any.
%        z0      - real vector, initial flow
%        ti      - real row vector, in multiple shooting case ti = [t0 t1 ... t_nbarc-1 tf]
%                    must be increasing.
%        options - struct vector, hampathset options
%        par     - real row vector, parameters given to hfun par=[] if no parameters
%
%    Outputs
%
%        tout    - real row vector, time at each integration step.
%        exphv   - real matrix, exphv(:,i) : flow at tout(i)
%        flag    - integer, flag should be 1 (ODE integrator output).
%
%-------------------------------------------------------------------------------------------
if(nargout~=3)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end

tspan=arg1;
z0=arg2;
if(nargin==nrhs0min)
ti =[tspan(1) tspan(end)];
options=arg3;
par=arg4;
elseif(nargin==nrhs0min+1)
ti=arg3;
options=arg4;
par=arg5;
end

[ tout, exphv, flag ]  = exphvfun_m(tspan,z0,par,options.TolOdeAbs,options.TolOdeRel, ...
                                        options.ODE,options.MaxStepsOde,ti,options.MaxStepSizeOde, ...
                                        options.TolScaling);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

