#include "fintrf.h"
C======================================================================
#if 0
C
C     hampath_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       [ parout,tiout,ziout,sout,viout,bc,btoc,dets,normS,ps,flag] =
C               hampath_m(parspan,zi,ti,options.ODE,options.ODEHam, ...
C               options.MaxSf,options.DispIter,options.Display,     ...
C               options.TolOdeAbs,options.TolOdeRel,                ...
C               options.TolOdeHamAbs,options.TolOdeHamRel,          ...
C               options.TolDenseAbs,options.MaxStepsOde,ntf,        ...
C               options.MaxSfunNorm,options.Derivative,             ...
C               options.MaxStepsOdeHam,options.MaxStepSizeOde,      ...
C               options.MaxStepSizeOdeHam,options.TolDenseRel,      ...
C               options.TolScaling);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C
C       .. Input Arguments ..
C
C       parspan         : double precision matrix
C       zi              : double precision matrix
C       ti              : double precision vector
C       ODE             : char*32
C       ODEHam          : char*32
C       MaxSf           : double precision
C       DispIter        : integer
C       Display         : char*32
C       TolOdeAbs       : double precision
C       TolOdeRel       : double precision
C       TolOdeHamAbs    : double precision
C       TolOdeHamRel    : double precision
C       TolDenseHamEnd  : double precision
C       MaxStepsOde     : integer
C       ntf             : integer
C       MaxSfunNorm     : double precision
C       Derivative      : char*32
C       MaxStepsOdeHam  : integer
C       MaxStepSizeOde  : double precision
C       MaxStepSizeOdeHam: double precision
C       TolDenseHamX    : double precision
C       TolScaling      : double precision
C       stopAtTurningPoint : char*32
C
C       .. Output Arguments
C
C       parout      : double precision matrix
C       tiout       : double precision vector
C       ziout       : double precision matrix
C       sout        : double precision vector
C       viout       : double precision matrix
C       bc          : double precision matrix
C       btoc        : double precision matrix
C       dets        : double precision vector
C       normS       : double precision vector
C       ps          : double precision vector
C       flag        : integer 
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0, icur
      parameter(nlhs0 = 11)
      parameter(nrhs0 = 11)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      mwPointer mxCreateString, mxIsInt8, mxCreateNumericArray
      integer mxGetString, mxIsNumeric, mxIsChar, statu
      integer*4 mxClassIDFromClassName

C   Arguments for mxCreateNumericArray
      integer*4 classid, complexflag
      mwSize ndim, dims(2)

C   Pointers to input/output arguments
      mwPointer parPr,ziPr,tiPr
      mwPointer derPr,dspPr,sapaPr,fftPr,odePr,odehPr
      mwPointer dwPr,iwPr,flagPr,nrmSPr,psPr
      mwPointer paroPr,tioPr,zioPr,soPr,vioPr,bcPr,btocPr,detsPr

C   Real, char or integer associated to pointer argument
      double precision dw(11),iwD(4),flagD
      integer ntf,ne,iw(4),flag
      character*32 ode,odeh,dsp,der,sapa,fft

C   Array information
      mwPointer mpar,npar,mzi,nzi,mti,nti
      mwPointer npath,nbif
      mwSize maxBuf
      parameter(maxBuf = 32)
      mwPointer strlen

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

      icur = 1
C       .. Get input : parspan
C       Columns = 2, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(icur))
      npar = mxGetN(prhs(icur))
      if(npar .ne. 2) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:nparspan',
     +                           'parspan must have exactly 2 columns.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:parspanNonNumeric',
     +                           'parspan must be a numeric array.')
      endif
      parPr = mxGetPr(prhs(icur))

      icur = 2
C       .. Get input : ti
C       Columns > 0, Rows = 1. isNumeric.
      mti = mxGetM(prhs(icur))
      nti = mxGetN(prhs(icur))
      if(mti .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:mti',
     +                           'ti must have one row.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(icur))

      icur = 3
C       .. Get input : zi
C       Columns > 0, Rows even. isNumeric.
      mzi = mxGetM(prhs(icur))
      nzi = mxGetN(prhs(icur))
      if(mod(mzi,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:mzi',
     +                           'zi must have even number of rows.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:ziNonNumeric',
     +                           'zi must be a numeric array.')
      endif
      ziPr = mxGetPr(prhs(icur))
      if(nti .ne. nzi+1) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:nti',
     +                          'ti must have one column more than zi.')
      end if

      icur = 4
C       .. Get input : Derivative
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DerivativeNonString',
     +                           'Derivative must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
         call mexErrMsgIdAndTxt('MATLAB:hampath:DerivativeNonRowVector',
     +                           'Derivative must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DerivativeOverMaxBuf',
     +                           'Derivative max length sticked out.')
      endif
      derPr = prhs(icur)
      statu = mxGetString(derPr,der,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DerivativeReadError',
     +                           'Derivative error reading string.')
      endif

      icur = 5
C       .. Get input : Display
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DisplayNonString',
     +                           'Display must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DisplayNonRowVector',
     +                           'Display must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DisplayOverMaxBuf',
     +                           'Display max length sticked out.')
      endif
      dspPr = prhs(icur)
      statu = mxGetString(dspPr,dsp,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DisplayReadError',
     +                           'Display error reading string.')
      endif

      icur = 6
C       .. Get input : DoSavePath
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DoSavePathNonString',
     +                           'DoSavePath must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
        call mexErrMsgIdAndTxt ('MATLAB:hampath:DoSavePathNonRowVector',
     +                           'DoSavePath must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DoSavePathOverMaxBuf',
     +                           'DoSavePath max length sticked out.')
      endif
      sapaPr = prhs(icur)
      statu = mxGetString(sapaPr,sapa,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:DoSavePathReadError',
     +                           'DoSavePath error reading string.')
      endif

      icur = 7
C       .. Get input: fft
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:fftNonString',
     +                      'FreeFinalTime must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:fftNonRowVector',
     +                      'FreeFinalTime must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:fftOverMaxBuf',
     +                      'FreeFinalTime max length sticked out.')
      endif
      fftPr = prhs(icur)
      statu = mxGetString(fftPr,fft,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:fftReadError',
     +                      'FreeFinalTime error reading string.')
      endif

      icur = 8
C       .. Get input : ODE
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeNonString',
     +                           'ODE must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeNonRowVector',
     +                           'ODE must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeOverMaxBuf',
     +                           'ODE max length sticked out.')
      endif
      odePr = prhs(icur)
      statu = mxGetString(odePr,ode,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeReadError',
     +                           'ODE error reading string.')
      endif

      icur = 9
C       .. Get input : ODEHam
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(icur)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeHamNonString',
     +                           'ODEHam must be a string.')
      endif
      if(mxGetM(prhs(icur)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeHamNonRowVector',
     +                           'ODEHam must be a row vector.')
      endif
      strlen = mxGetM(prhs(icur))*mxGetN(prhs(icur))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeHamOverMaxBuf',
     +                           'ODEHam max length sticked out.')
      endif
      odehPr = prhs(icur)
      statu = mxGetString(odehPr,odeh,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:odeHamReadError',
     +                           'ODEHam error reading string.')
      endif

      icur = 10
C       .. Get input: iwork
C       Columns = 4, Rows = 1. isNumeric.
      if(mxGetM(prhs(icur)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:iwork',
     +                           'iwork must have one row.')
      end if
      if(mxGetN(prhs(icur)) .ne. 4) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:iwork',
     +                          'iwork must have 4 columns.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:iworkNonNumeric',
     +                           'iwork must be a numeric array.')
      endif
      iwPr = mxGetPr(prhs(icur))
      call mxCopyPtrToReal8(iwPr,iwD,4)
      iw   = nint(iwD)

      icur = 11
C       .. Get input: dwork
C       Columns = 11, Rows = 1. isNumeric.
      if(mxGetM(prhs(icur)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:dwork',
     +                           'dwork must have one row.')
      end if
      if(mxGetN(prhs(icur)) .ne. 11) then
          call mexErrMsgIdAndTxt ('MATLAB:hampath:dwork',
     +                          'dwork must have 11 columns.')
      end if
      if(mxIsNumeric(prhs(icur)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hampath:dworkNonNumeric',
     +                           'dwork must be a numeric array.')
      endif
      dwPr = mxGetPr(prhs(icur))
      call mxCopyPtrToReal8(dwPr,dw,11)

C-----------------------------------------------------------------------

C    .. call important init ..
      call initTtfNe(fft,nzi,mzi/2,ntf,ne)

C    .. Create output flag from hampath ..
C      classid  = mxClassIDFromClassName('int8')
C      complexflag = 0
C      ndim     = 1
C      dims(1)  = 1
C      plhs(11) = mxCreateNumericArray(ndim,dims,classid,complexflag)
      plhs(11) = mxCreateDoubleMatrix(1, 1, 0)
      flagPr   = mxGetPr(plhs(11))

C     .. Computation .. Give pointer argument for matrix or vector
      call hampathmexint(mpar,npar,parPr,mzi,nzi,ziPr,mti,nti,tiPr,
     +      der,dsp,sapa,ode,odeh,dw,iw,npath,nbif,flag,ne,ntf)

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(mpar, npath, 0)
      paroPr  = mxGetPr(plhs(1))

      plhs(2) = mxCreateDoubleMatrix(1, (nzi+1)*npath, 0)
      tioPr   = mxGetPr(plhs(2))

      plhs(3) = mxCreateDoubleMatrix(mzi, nzi*npath, 0)
      zioPr   = mxGetPr(plhs(3))

      plhs(4) = mxCreateDoubleMatrix(1, npath, 0)
      soPr    = mxGetPr(plhs(4))

      plhs(5) = mxCreateDoubleMatrix(ne+1, npath, 0)
      vioPr   = mxGetPr(plhs(5))

      plhs(8) = mxCreateDoubleMatrix(1, npath, 0)
      detsPr  = mxGetPr(plhs(8))

      plhs(9) = mxCreateDoubleMatrix(1, npath, 0)
      nrmSPr  = mxGetPr(plhs(9))

      plhs(10) = mxCreateDoubleMatrix(1, npath, 0)
      psPr     = mxGetPr(plhs(10))

      call getpathmex(tiPr,nzi,mzi,ne,ntf,npath,mpar,
     +          paroPr,tioPr,zioPr,vioPr,soPr,detsPr,nrmsPr,psPr)

      plhs(6) = mxCreateDoubleMatrix(ne+2, nbif, 0)
      bcPr    = mxGetPr(plhs(6))

      plhs(7) = mxCreateDoubleMatrix(2*(ne+1), nbif, 0)
      btocPr  = mxGetPr(plhs(7))

      call getbifmex(ne,mzi,nbif,bcPr,btocPr)

      flagD = dble(flag)
      call mxCopyReal8ToPtr(flagD, flagPr, 1)

      return
      end

C-----------------------------------------------------------------------
C     Computational routine
      subroutine hampathmexint(mpar,npar,parPr,mzi,nzi,ziPr,mti,nti,
     +   tiPr,der,dsp,sapa,ode,odeh,dw,iw,npath,nbif,flag,ne,ntf)
      implicit none

      mwPointer parPr,ziPr,tiPr
      double precision dw(11)
      integer flag,iw(4),ne,ntf
      character*32 ode,odeh,dsp,der,sapa
      mwPointer mpar,npar,mzi,nzi,mti,nti
      mwPointer npath,nbif

      double precision ti(nzi+1),zi(mzi,nzi),par(mpar,npar)

      call mxCopyPtrToReal8(tiPr,ti,nzi+1)
      call mxCopyPtrToReal8(ziPr,zi,mzi*nzi)
      call mxCopyPtrToReal8(parPr,par,mpar*npar)

      call hampathint(nzi,ti,mzi/2,zi,mpar,npar,par,ne,ntf,npath,nbif,
     *              flag,iw,dw,der,dsp,ode,odeh,sapa)

      return
      end

      subroutine getpathmex(tiPr,nzi,mzi,ne,ntf,npath,mpar,paroPr,
     +                  tioPr,zioPr,vioPr,soPr,detsPr,nrmsPr,psPr)
      implicit none

      mwPointer tiPr
      mwPointer mpar,mzi,nzi
      mwPointer npath
      integer ne,ntf

      mwPointer paroPr,tioPr,zioPr,soPr,vioPr,detsPr,nrmSPr,psPr

      double precision ti(nzi+1)
      double precision paro(mpar,npath),tio((nzi+1)*npath),
     +              zio(mzi,nzi*npath),so(npath),vio(ne+1,npath),
     +              dets(npath),nrms(npath),ps(npath)

      call mxCopyPtrToReal8(tiPr,ti,nzi+1)
      call getpath(ti(1),ti(nzi+1),nzi,mzi/2,ne,ntf,npath,mpar,paro,
     +              tio,zio,vio,so,dets,nrms,ps)

      call mxCopyReal8ToPtr(paro,paroPr,mpar*npath)
      call mxCopyReal8ToPtr(tio,tioPr,(nzi+1)*npath)
      call mxCopyReal8ToPtr(zio,zioPr,mzi*nzi*npath)
      call mxCopyReal8ToPtr(so,soPr,npath)
      call mxCopyReal8ToPtr(vio,vioPr,(ne+1)*npath)
      call mxCopyReal8ToPtr(dets,detsPr,npath)
      call mxCopyReal8ToPtr(nrms,nrmsPr,npath)
      call mxCopyReal8ToPtr(ps,psPr,npath)

      return
      end

      subroutine getbifmex(ne,mzi,nbif,bcPr,btocPr)
      implicit none

      mwPointer mzi,nbif,bcPr,btocPr
      integer ne

      double precision bc(ne+2,nbif),btoc(2*(ne+1),nbif)

      call getbif(mzi/2,ne,nbif,bc,btoc)

      call mxCopyReal8ToPtr(bc,bcPr,(ne+2)*nbif)
      call mxCopyReal8ToPtr(btoc,btocPr,2*(ne+1)*nbif)

      return
      end

