function h = hfun(arg1,arg2,arg3,arg4)
%-------------------------------------------------------------------------------------------
%
%    hfun (needs hfun.f90)
%
%    Description
%
%        Computes the Hamiltonian.
%
%-------------------------------------------------------------------------------------------
%
%    Matlab Usage
%
%        h = hfun(t, z,     par) : single arc
%        h = hfun(t, z, ti, par) : multiple arcs
%
%    Inputs
%
%        t    -  real row vector, t(i) = i-th time
%        z    -  real matrix, z(:,i) = i-th state and costate
%        ti   -  real row vector, in multiple shooting case, ti = [t0 t1 ... t_nbarc-1 tf]
%        par  -  real row vector, parameters given to hfun par=[] if no parameters
%
%    Outputs
%
%        h    -  real row vector, Hamiltonian at time(s) t
%
%-------------------------------------------------------------------------------------------
if(nargout~=1)
error('wrong number of outputs: try help');
end

nrhs0min = 3;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end

t=arg1;
z=arg2;
if(nargin==nrhs0min)
ti =[t(1) t(end)];
par=arg3;
elseif(nargin==nrhs0min+1)
ti=arg3;
par=arg4;
end

h = hfun_m(t, z, ti, par);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!
