#include "fintrf.h"
C======================================================================
#if 0
C
C     hvfun_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       hv = hvfun_m(t, z, ti, par);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C
C       .. Input Arguments ..
C
C       t           : double precision vector
C       z           : double precision matrix
C       ti          : double precision vector
C       par         : double precision vector
C
C       .. Output Arguments
C
C       hv          : double precision matrix
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 1)
      parameter(nrhs0 = 4)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      integer mxIsNumeric

C   Pointers to input/output arguments
      mwPointer tPr,zPr,tiPr,parPr
      mwPointer hvPr

C   Array information
      mwPointer nt,mz,nz,nti,mpar

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input 1 : t
C       Columns > 0, Rows = 1. isNumeric.
      nt = mxGetN(prhs(1))
      if(mxGetM(prhs(1)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hvfun:mt',
     +                           't must have one row.')
      end if
      if(mxIsNumeric(prhs(1)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:tNonNumeric',
     +                           't must be a numeric array.')
      endif
      tPr = mxGetPr(prhs(1))

C       .. Get input 2 : z
C       Columns > 0, Rows > 0. isNumeric.
      mz = mxGetM(prhs(2))
      nz = mxGetN(prhs(2))
      if(mod(mz,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:hvfun:mz',
     +                           'z must have even number of rows.')
      end if
      if(nz .ne. nt) then
          call mexErrMsgIdAndTxt ('MATLAB:hvfun:nz',
     +              'z must have the same number of rows than t.')
      end if
      if(mxIsNumeric(prhs(2)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:zNonNumeric',
     +                           'z must be a numeric array.')
      endif
      zPr = mxGetPr(prhs(2))

C       .. Get input 3 : ti
C       Columns > 0, Rows = 1. isNumeric.
      nti = mxGetN(prhs(3))
      if(mxGetM(prhs(3)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hvfun:mti',
     +                           'ti must have one row.')
      end if
      if(mxIsNumeric(prhs(3)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(3))

C       .. Get input 4 : par
C       Columns = 1, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(4))
      if(mxGetN(prhs(4)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:hvfun:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(4)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:hvfun:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(4))

C-----------------------------------------------------------------------

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(mz, nt, 0)
      hvPr    = mxGetPr(plhs(1))

C     .. Computation ..
      call hvfunmexint(tPr,mz,zPr,nti,tiPr,mpar,parPr,nt,hvPr)

      RETURN
      END

C-----------------------------------------------------------------------
C     Computational routine
      subroutine hvfunmexint(tPr,mz,zPr,nti,tiPr,mpar,parPr,nt,hvPr)

      implicit none

      mwPointer tPr,zPr,tiPr,parPr
      mwPointer hvPr
      mwPointer nt,mz,nz,nti,mpar

      double precision t(nt),z(mz,nt),ti(nti),par(mpar),hv(mz,nt)

      call mxCopyPtrToReal8(tPr,t,nt)
      call mxCopyPtrToReal8(zPr,z,mz*nt)
      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call hvfunint(t,mz/2,z,nti-1,ti,mpar,par,nt,hv)

      call mxCopyReal8ToPtr(hv,hvPr,mz*nt)

      return
      end


