      SUBROUTINE MEXFUNCTION(NLHS, PLHS, NRHS, PRHS)
      IMPLICIT NONE
C     Written on Thu 12 Jun 2008 14:57:08 CEST
C     by Jean-Baptiste Caillau - Math. Institute, Bourgogne Univ.
C
C     Function [ x, y, iflag, nfev ] = hybrd_m(nlefun, x0, xTol,
C     MaxFev, ml, mu, EpsFcn, diag, mode, factor, p1, ...).

C     .. Global ..
#include "fintrf.h"
      INTEGER          NVINMAX
      PARAMETER       (NVINMAX = 32)
      INTEGER          NVIN
      MWPOINTER        MXVIN(NVINMAX)
      INTEGER          NFEV2
      CHARACTER*32     NLEFUN

      COMMON           /CNVIN/NVIN
      COMMON           /CMXVIN/MXVIN
      COMMON           /CNFEV2/NFEV2
      COMMON           /CNLEFUN/NLEFUN

C     .. Arguments ..
C     NLHS             (input) INTEGER
C                      Number of outputs
C     PLHS             (input) MWPOINTER array
C                      Outputs
C     NRHS             (input) INTEGER
C                      Number of inputs
C     PRHS             (input) MWPOINTER array
C                      Inputs
      INTEGER          NLHS
      INTEGER          NRHS
      MWPOINTER        PLHS(*)
      MWPOINTER        PRHS(*)

C     .. Local declarations ..
      INTEGER          DBSIZE
      PARAMETER       (DBSIZE = 8)
      INTEGER          NLHS0
      PARAMETER       (NLHS0 =  4)
      INTEGER          NRHS0
      PARAMETER       (NRHS0 = 10)
      MWPOINTER        PNLEFUN
      MWPOINTER        PX0
      MWPOINTER        PXTOL
      MWPOINTER        PMAXFEV
      MWPOINTER        PML
      MWPOINTER        PMU
      MWPOINTER        PEPSFCN
      MWPOINTER        PDIAG
      MWPOINTER        PMODE
      MWPOINTER        PFACTOR
      MWPOINTER        PX
      MWPOINTER        PY
      MWPOINTER        PIFLAG
      MWPOINTER        PNFEV
      INTEGER          MAXFEV
      INTEGER          ML
      INTEGER          MU
      INTEGER          MODE
      INTEGER          NPRINT
      INTEGER          IFLAG
      INTEGER          NFEV
      INTEGER          N
      INTEGER          LR
      INTEGER          RET
      INTEGER          I
      INTEGER          MXGETSTRING
      MWPOINTER        MXGETPR
      INTEGER          MXGETM
      INTEGER          MXGETN
      MWPOINTER        MXCREATEDOUBLEMATRIX
      DOUBLE PRECISION MMAXFEV
      DOUBLE PRECISION MML
      DOUBLE PRECISION MMU
      DOUBLE PRECISION MMODE
      DOUBLE PRECISION IIFLAG
      DOUBLE PRECISION NNFEV

C     .. Check inputs ..
      IF (NRHS.LT.NRHS0) THEN
        CALL MEXERRMSGTXT('Not enough input arguments.')
      ELSEIF (NRHS.GT.(NRHS0 + NVINMAX)) THEN
        CALL MEXERRMSGTXT('Too many input arguments.')
      ENDIF

C     .. Check outputs ..
      IF (NLHS.LT.NLHS0) THEN
        CALL MEXERRMSGTXT('Not enough output arguments.')
      ELSEIF (NLHS.GT.NLHS0) THEN
        CALL MEXERRMSGTXT('Too many output arguments.')
      ENDIF

C     .. Get inputs ..
      PNLEFUN = PRHS(1)
      RET = MXGETSTRING(PNLEFUN, NLEFUN, 32)
      IF (RET.NE.0) THEN
        CALL MEXERRMSGTXT('nlefun must be a string.')
      END IF

      IF (MXGETN(PRHS(2)).NE.1) THEN
        CALL MEXERRMSGTXT('x0 must have one column.')
      ENDIF
      PX0 = MXGETPR(PRHS(2))
      N   = MXGETM(PRHS(2))

      IF (MXGETM(PRHS(3)).NE.1) THEN
        CALL MEXERRMSGTXT('xTol must be scalar.')
      ELSEIF (MXGETN(PRHS(3)).NE.1) THEN
        CALL MEXERRMSGTXT('xTol must be scalar.')
      ENDIF
      PXTOL = MXGETPR(PRHS(3))

      IF (MXGETM(PRHS(4)).NE.1) THEN
        CALL MEXERRMSGTXT('MaxFev must be scalar.')
      ELSEIF (MXGETN(PRHS(4)).NE.1) THEN
        CALL MEXERRMSGTXT('MaxFev must be scalar.')
      ENDIF
      PMAXFEV = MXGETPR(PRHS(4))
      CALL MXCOPYPTRTOREAL8(PMAXFEV, MMAXFEV, 1)
      MAXFEV = NINT(MMAXFEV)

      IF (MXGETM(PRHS(5)).NE.1) THEN
        CALL MEXERRMSGTXT('ml must be scalar.')
      ELSEIF (MXGETN(PRHS(5)).NE.1) THEN
        CALL MEXERRMSGTXT('ml must be scalar.')
      ENDIF
      PML = MXGETPR(PRHS(5))
      CALL MXCOPYPTRTOREAL8(PML, MML, 1)
      ML = NINT(MML)

      IF (MXGETM(PRHS(6)).NE.1) THEN
        CALL MEXERRMSGTXT('mu must be scalar.')
      ELSEIF (MXGETN(PRHS(6)).NE.1) THEN
        CALL MEXERRMSGTXT('mu must be scalar.')
      ENDIF
      PMU = MXGETPR(PRHS(6))
      CALL MXCOPYPTRTOREAL8(PMU, MMU, 1)
      MU = NINT(MMU)

      IF (MXGETM(PRHS(7)).NE.1) THEN
        CALL MEXERRMSGTXT('EpsFcn must be scalar.')
      ELSEIF (MXGETN(PRHS(7)).NE.1) THEN
        CALL MEXERRMSGTXT('EpsFcn must be scalar.')
      ENDIF
      PEPSFCN = MXGETPR(PRHS(7))

      IF (MXGETN(PRHS(8)).NE.1) THEN
        CALL MEXERRMSGTXT('diag must have one column.')
      ELSEIF (MXGETM(PRHS(8)).NE.N) THEN
        CALL MEXERRMSGTXT('diag must have n rows.')
      ENDIF
      PDIAG = MXGETPR(PRHS(8))

      IF (MXGETM(PRHS(9)).NE.1) THEN
        CALL MEXERRMSGTXT('mode must be scalar.')
      ELSEIF (MXGETN(PRHS(9)).NE.1) THEN
        CALL MEXERRMSGTXT('mode must be scalar.')
      ENDIF
      PMODE = MXGETPR(PRHS(9))
      CALL MXCOPYPTRTOREAL8(PMODE, MMODE, 1)
      MODE = NINT(MMODE)

      IF (MXGETM(PRHS(10)).NE.1) THEN
        CALL MEXERRMSGTXT('Factor must be scalar.')
      ELSEIF (MXGETN(PRHS(10)).NE.1) THEN
        CALL MEXERRMSGTXT('Factor must be scalar.')
      ENDIF
      PFACTOR = MXGETPR(PRHS(10))

      NVIN = NRHS - NRHS0
      DO I = 1, NVIN
        MXVIN(I) = PRHS(NRHS0 + I)
      END DO

C     .. Create outputs ..
      PLHS(1) = MXCREATEDOUBLEMATRIX(N, 1, 0)
      PX      = MXGETPR(PLHS(1))

      PLHS(2) = MXCREATEDOUBLEMATRIX(N, 1, 0)
      PY      = MXGETPR(PLHS(2))

      PLHS(3) = MXCREATEDOUBLEMATRIX(1, 1, 0)
      PIFLAG  = MXGETPR(PLHS(3))

      PLHS(4) = MXCREATEDOUBLEMATRIX(1, 1, 0)
      PNFEV   = MXGETPR(PLHS(4))

C     .. Computation ..
C      CALL MXCOPYPTRTOREAL8(PX0, %val(PX), N)
      NPRINT = -1 ! no special printing
      IFLAG  = -1 ! normal call
      LR     = N*(N+1)/2
C      PFJAC  = MXCALLOC(N*N, DBSIZE)
C      PR     = MXCALLOC( LR, DBSIZE)
C      PQTF   = MXCALLOC(  N, DBSIZE)
C      PWA1   = MXCALLOC(  N, DBSIZE)
C      PWA2   = MXCALLOC(  N, DBSIZE)
C      PWA3   = MXCALLOC(  N, DBSIZE)
C      PWA4   = MXCALLOC(  N, DBSIZE)

      NFEV2  = 0

      WRITE(*, '( a6, $)') 'Calls'
      WRITE(*, '(a23, $)') '|f(x)|               '
      WRITE(*, '(a23, $)') '|x|                  '
C      WRITE(*, '(a11   )') 'x        '
      WRITE(*,*) ''

      call hybrdmex(N,PX0,PX,PY,PXTOL,MAXFEV,ML,MU,
     +              PEPSFCN,PDIAG,MODE,PFACTOR,NPRINT,IFLAG,NFEV,LR)

C      call hybrdmex(N,PX0,PX,PY,PXTOL,MAXFEV,ML,MU,
C     +              PEPSFCN,PDIAG,MODE,PFACTOR,NPRINT,IFLAG,NFEV,
C     +              PFJAC,PR,LR,PQTF,PWA1,PWA2,PWA3,PWA4)

      IIFLAG = DBLE(IFLAG)
      CALL MXCOPYREAL8TOPTR(IIFLAG, PIFLAG, 1)

      NNFEV = DBLE(NFEV)
      CALL MXCOPYREAL8TOPTR(NNFEV, PNFEV, 1)

      RETURN
      END

C-----------------------------------------------------------------------
C     Computational routine
      subroutine hybrdmex(N,PX0,PX,PY,PXTOL,MAXFEV,ML,MU,
     +              PEPSFCN,PDIAG,MODE,PFACTOR,NPRINT,IFLAG,NFEV,LR)

      implicit none

      MWPOINTER        PX0
      MWPOINTER        PXTOL
      MWPOINTER        PMAXFEV
      MWPOINTER        PEPSFCN
      MWPOINTER        PDIAG
      MWPOINTER        PFACTOR
      MWPOINTER        PX
      MWPOINTER        PY
      INTEGER          MAXFEV
      INTEGER          ML
      INTEGER          MU
      INTEGER          MODE
      INTEGER          NPRINT
      INTEGER          IFLAG
      INTEGER          NFEV
      INTEGER          N
      INTEGER          LR
      EXTERNAL         HYBRDFUN

      double precision X(N),Y(N),XTOL,EPSFCN,DIAG(N),FACTOR,FJAC(N,N)
      double precision QTF(N),WA1(N),WA2(N),WA3(N),WA4(N),R(LR)

      call mxCopyPtrToReal8(PX0,X,N)
      call mxCopyPtrToReal8(PXTOL,XTOL,1)
      call mxCopyPtrToReal8(PEPSFCN,EPSFCN,1)
      call mxCopyPtrToReal8(PDIAG,DIAG,N)
      call mxCopyPtrToReal8(PFACTOR,FACTOR,1)

      call hybrd(HYBRDFUN,N,X,Y,XTOL,MAXFEV,ML,MU,EPSFCN,DIAG,
     +          MODE,FACTOR,NPRINT,
     +          IFLAG,NFEV,FJAC,N,R,LR,QTF,WA1,WA2,WA3,WA4)

      call mxCopyReal8ToPtr(X,PX,N)
      call mxCopyReal8ToPtr(Y,PY,N)

      return
      end

