function [s, flag]  = sfun(ti,zi,options,par)
%-------------------------------------------------------------------------------------------
%
%    sfun (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Computes the shooting function
%
%    Options used
%
%        FreeFinalTime, MaxStepsOde, MaxStepSizeOde, ODE, TolOdeAbs, TolOdeRel, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [s, flag] = sfun(ti,zi,options,par)
%
%    Inputs
%
%        ti      - real row vector,
%                    ti = [t0 tf]                    : simple shooting
%                    ti = [t0 t1 ... t_{nbarc-1} tf] : multiple shooting
%        zi      - real matrix, shooting variable,
%                    zi = [z(t0) ... z(t_{nbarc-1})]
%        options - struct, hampathset options
%        par     - real row vector, par in hfun and efun, par=[] if no parameters
%
%    Outputs
%
%        s       - real vector, shooting value
%        flag    - integer, should be 1 (ode output).
%
%-------------------------------------------------------------------------------------------
if(nargout~=2)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end

[s, flag] = sfun_m(ti,zi,par,options.TolOdeAbs,options.TolOdeRel, ...
                    options.ODE,options.MaxStepsOde,options.FreeFinalTime,options.MaxStepSizeOde,options.TolScaling);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

