#include "fintrf.h"
C======================================================================
#if 0
C
C     sfun_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       [s, flag] = sfun_m(ti,zi,par,options.TolOdeAbs,options.TolOdeRel,
C            options.ODE,options.MaxStepsOde,ntf,options.MaxStepSizeOde,
C            options.TolScaling);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C                       Inputs
C
C       .. Input Arguments ..
C
C       ti              : double precision vector
C       zi              : double precision matrix
C       par             : double precision vector
C       TolOdeAbs       : double precision
C       TolOdeRel       : double precision
C       ODE             : char*32
C       MaxStepsOde     : integer
C       ntf             : integer
C       MaxStepSizeOde  : double precision
C       TolScaling      : double precision
C
C       .. Output Arguments
C
C       s           : double precision vector
C       flag        : integer
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 2)
      parameter(nrhs0 = 10)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      mwPointer mxCreateString, mxIsInt8, mxCreateNumericArray
      integer mxGetString, mxIsNumeric, mxIsChar, statu
      integer*4 mxClassIDFromClassName

C   Arguments for mxCreateNumericArray
      integer*4 classid, complexflag
      mwSize ndim, dims(2)

C   Pointers to input/output arguments
      mwPointer tiPr,ziPr,parPr,atPr,rtPr,odePr,msPr,fftPr
      mwPointer sPr,flagPr,mssPr,tscPr

C   Real, char or integer associated to pointer argument
      double precision at,rt,msD,flagD,mss,tsc
      integer ms,flag,ntf,ne
      character*32 ode,fft

C   Array information
      mwPointer nti,mzi,nzi,mpar
      mwSize maxBuf
      parameter(maxBuf = 32)
      mwPointer strlen

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input 1 : ti
C       Columns > 0, Rows = 1. isNumeric.
      nti = mxGetN(prhs(1))
      if(mxGetM(prhs(1)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:mti',
     +                           'ti must have one row.')
      end if
      if(nti .le. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:nti',
     +                           'Check that ti=[t0 t1 tm-1 tf].')
      end if
      if(mxIsNumeric(prhs(1)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(1))

C       .. Get input 2 : zi
C       Columns > 0, Rows > 0. isNumeric.
      mzi = mxGetM(prhs(2))
      nzi = mxGetN(prhs(2))
      if(mod(mzi,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:mzi',
     +                       'zi must have an even number of rows.')
      end if
      if(mxIsNumeric(prhs(2)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:ziNonNumeric',
     +                           'zi must be a numeric array.')
      endif
      ziPr = mxGetPr(prhs(2))

C       .. Get input 3 : par
C       Columns = 1, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(3))
      if(mxGetN(prhs(3)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(3)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(3))


C       .. Get input 4 : TolOdeAbs
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(4))*mxGetN(prhs(4)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:TolOdeAbs',
     +                           'TolOdeAbs must be scalar.')
      end if
      if(mxIsNumeric(prhs(4)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:TolOdeAbsNonNumeric',
     +                           'TolOdeAbs must be a numeric scalar.')
      endif
      atPr = mxGetPr(prhs(4))
      call mxCopyPtrToReal8(atPr,at,1)

C       .. Get input 5 : TolOdeRel
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(5))*mxGetN(prhs(5)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:TolOdeRel',
     +                           'TolOdeRel must be scalar.')
      end if
      if(mxIsNumeric(prhs(5)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:TolOdeRelNonNumeric',
     +                           'TolOdeRel must be a numeric scalar.')
      endif
      rtPr = mxGetPr(prhs(5))
      call mxCopyPtrToReal8(rtPr,rt,1)

C       .. Get input 6 : ODE
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(6)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:odeNonString',
     +                           'ODE must be a string.')
      endif
      if(mxGetM(prhs(6)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:odeNonRowVector',
     +                           'ODE must be a row vector.')
      endif
      strlen = mxGetM(prhs(6))*mxGetN(prhs(6))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:odeOverMaxBuf',
     +                           'ODE max length sticked out.')
      endif
      odePr = prhs(6)
      statu = mxGetString(odePr,ode,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:odeReadError',
     +                           'ODE error reading string.')
      endif

C       .. Get input 7 : MaxStepsOde
C       Columns = 1, Rows = 1. isInt8.
      if(mxGetM(prhs(7))*mxGetN(prhs(7)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:MaxStepsOde',
     +                            'MaxStepsOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(7)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:MaxStepsOdeNonInt',
     +                           'MaxStepsOde must be an integer.')
      endif
      msPr = mxGetPr(prhs(7))
      call mxCopyPtrToReal8(msPr,msD,1)
      ms = nint(msD)

C       .. Get input 8 : fft
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(8)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:fftNonString',
     +                      'FreeFinalTime must be a string.')
      endif
      if(mxGetM(prhs(8)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:fftNonRowVector',
     +                      'FreeFinalTime must be a row vector.')
      endif
      strlen = mxGetM(prhs(8))*mxGetN(prhs(8))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:fftOverMaxBuf',
     +                      'FreeFinalTime max length sticked out.')
      endif
      fftPr = prhs(8)
      statu = mxGetString(fftPr,fft,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:fftReadError',
     +                      'FreeFinalTime error reading string.')
      endif


C       .. Get input 9 : MaxStepSizeOde
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(9))*mxGetN(prhs(9)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:MaxStepSizeOde',
     +                           'MaxStepSizeOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(9)) .eq. 0) then
      call mexErrMsgIdAndTxt ('MATLAB:sfun:MaxStepSizeOdeNonNumeric',
     +                      'MaxStepSizeOde must be a numeric scalar.')
      endif
      mssPr = mxGetPr(prhs(9))
      call mxCopyPtrToReal8(mssPr,mss,1)

C       .. Get input 10 : TolScaling
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(10))*mxGetN(prhs(10)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sfun:TolScaling',
     +                           'TolScaling must be scalar.')
      end if
      if(mxIsNumeric(prhs(10)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sfun:TolScalingNonNumeric',
     +                           'TolScaling must be a numeric scalar.')
      endif
      tscPr = mxGetPr(prhs(10))
      call mxCopyPtrToReal8(tscPr,tsc,1)

C-----------------------------------------------------------------------

C    .. call important init ..
      call initTtfNe(fft,nzi,mzi/2,ntf,ne)

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(ne, 1, 0)
      sPr     = mxGetPr(plhs(1))

C    .. Create output flag ..
C      classid  = mxClassIDFromClassName('int8')
C      complexflag = 0
C      ndim     = 1
C      dims(1)  = 1
C      plhs(2) = mxCreateNumericArray(ndim,dims,classid,complexflag)
      plhs(2) = mxCreateDoubleMatrix(1, 1, 0)
      flagPr  = mxGetPr(plhs(2))

C     .. Computation ..
      call sfunmexint(nti,tiPr,mzi,nzi,ziPr,mpar,parPr,at,rt,ode,
     +                      ms,ne,sPr,flag,mss,tsc)

      flagD = dble(flag)
      call mxCopyReal8ToPtr(flagD, flagPr, 1)

      return
      end

C-----------------------------------------------------------------------
C     Computational routine
      subroutine sfunmexint(nti,tiPr,mzi,nzi,ziPr,mpar,parPr,at,rt,ode,
     +                      ms,ne,sPr,flag,mss,tsc)

      implicit none

      mwPointer tiPr,ziPr,parPr
      mwPointer sPr
      double precision at,rt,mss,tsc
      integer ms,flag,ne
      character*32 ode
      mwPointer nti,mzi,nzi,mpar

      double precision ti(nti),zi(mzi,nzi),par(mpar)
      double precision s(ne)

      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(ziPr,zi,mzi*nzi)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call sfunint(nzi,ti,mzi/2,zi,mpar,par,ne,s,flag,at,rt,ode,ms,
     *              mss,tsc)

      call mxCopyReal8ToPtr(s,sPr,ne)

      return
      end

