function [fjac, flag] = sjac(ti,zi,options,par,indexpar)
%-------------------------------------------------------------------------------------------
%
%    sjac (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Computes the Jacobian of the shooting-homotopic function
%
%    Options used
%
%        Derivative, FreeFinalTime, MaxStepsOde, MaxStepSizeOde, ODE, TolOdeAbs, 
%        TolOdeRel, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [fjac, flag] = sjac(ti,zi,options,par)
%        [fjac, flag] = sjac(ti,zi,options,par,indexpar)
%
%    Inputs
%
%        ti      - real row vector,
%                    ti = [t0 tf]                    : simple shooting
%                    ti = [t0 t1 ... t_{nbarc-1} tf] : multiple shooting
%        zi      - real matrix, shooting variable,
%                    zi = [z(t0) ... z(t_{nbarc-1})]
%        options - struct, hampathset options
%        par     - real row vector, par in hfun and efun, par=[] if no parameters
%        indexpar- integer vector, index of parameters for which the derivative is computed
%
%    Outputs
%
%        fjac    - real matrix, jacobian of the homotopic function
%        flag    - integer, should be 1 (ode output).
%
%-------------------------------------------------------------------------------------------
if(nargout~=2)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end
if(nargin==4)
indexpar=[];
else
nmin=min(indexpar);
nmax=max(indexpar);
if(nmin<1)
error('indexpar must contain values between 1 and length(par)');
end
if(nmax>length(par))
error('indexpar must contain values between 1 and length(par)');
end
end

[fjac, flag] = sjac_m(ti,zi,par,options.TolOdeAbs,options.TolOdeRel,options.ODE, ...
                        options.MaxStepsOde,options.FreeFinalTime,indexpar,options.Derivative,options.MaxStepSizeOde, ...
                        options.TolScaling);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!



