#include "fintrf.h"
C======================================================================
#if 0
C
C     sjac_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       [fjac, flag] = sjac_m(ti,zi,par,options.TolOdeAbs,          ...
C               options.TolOdeRel,options.ODE,options.MaxStepsOde,  ...
C               ntf,indexpar,options.Derivative,                    ...
C               options.MaxStepSizeOde,options.TolScaling);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C                       Inputs
C
C       .. Input Arguments ..
C
C       ti              : double precision vector
C       zi              : double precision matrix
C       par             : double precision vector
C       TolOdeAbs       : double precision
C       TolOdeRel       : double precision
C       ODE             : char*32
C       MaxStepsOde     : integer
C       ntf             : integer
C       indexpar        : integer
C       MaxStepSizeOde  : double precision
C       TolScaling      : double precision
C
C       .. Output Arguments
C
C       fjac        : double precision matrix
C       flag        : integer
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 2)
      parameter(nrhs0 = 12)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      mwPointer mxCreateString, mxIsInt8, mxCreateNumericArray
      integer mxGetString, mxIsNumeric, mxIsChar, statu
      integer*4 mxClassIDFromClassName

C   Arguments for mxCreateNumericArray
      integer*4 classid, complexflag
      mwSize ndim, dims(2)

C   Pointers to input/output arguments
      mwPointer tiPr,ziPr,parPr,atPr,rtPr,odePr,msPr,iparPr,derPr,fftPr
      mwPointer sPr,flagPr,mssPr,tscPr

C   Real, char or integer associated to pointer argument
      double precision at,rt,msD,flagD,mss,tsc
      integer ms,flag,ntf,ne,nl
      character*32 ode,der,fft

C   Array information
      mwPointer nti,mzi,nzi,mpar,mipar
      mwSize maxBuf
      parameter(maxBuf = 32)
      mwPointer strlen

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input 1 : ti
C       Columns > 0, Rows = 1. isNumeric.
      nti = mxGetN(prhs(1))
      if(mxGetM(prhs(1)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:mti',
     +                           'ti must have one row.')
      end if
      if(nti .le. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:nti',
     +                           'Check that ti=[t0 t1 tm-1 tf].')
      end if
      if(mxIsNumeric(prhs(1)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:tiNonNumeric',
     +                           'ti must be a numeric array.')
      endif
      tiPr = mxGetPr(prhs(1))

C       .. Get input 2 : zi
C       Columns > 0, Rows > 0. isNumeric.
      mzi = mxGetM(prhs(2))
      nzi = mxGetN(prhs(2))
      if(mod(mzi,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:mzi',
     +                       'zi must have an even number of rows.')
      end if
      if(mxIsNumeric(prhs(2)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:ziNonNumeric',
     +                           'zi must be a numeric array.')
      endif
      ziPr = mxGetPr(prhs(2))

C       .. Get input 3 : par
C       Columns = 1, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(3))
      if(mxGetN(prhs(3)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(3)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(3))


C       .. Get input 4 : TolOdeAbs
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(4))*mxGetN(prhs(4)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:TolOdeAbs',
     +                           'TolOdeAbs must be scalar.')
      endif
      if(mxIsNumeric(prhs(4)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:TolOdeAbsNonNumeric',
     +                           'TolOdeAbs must be a numeric scalar.')
      endif
      atPr = mxGetPr(prhs(4))
      call mxCopyPtrToReal8(atPr,at,1)

C       .. Get input 5 : TolOdeRel
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(5))*mxGetN(prhs(5)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:TolOdeRel',
     +                           'TolOdeRel must be scalar.')
      endif
      if(mxIsNumeric(prhs(5)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:TolOdeRelNonNumeric',
     +                           'TolOdeRel must be a numeric scalar.')
      endif
      rtPr = mxGetPr(prhs(5))
      call mxCopyPtrToReal8(rtPr,rt,1)

C       .. Get input 6 : ODE
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(6)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:odeNonString',
     +                           'ODE must be a string.')
      endif
      if(mxGetM(prhs(6)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:odeNonRowVector',
     +                           'ODE must be a row vector.')
      endif
      strlen = mxGetM(prhs(6))*mxGetN(prhs(6))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:odeOverMaxBuf',
     +                           'ODE max length sticked out.')
      endif
      odePr = prhs(6)
      statu = mxGetString(odePr,ode,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:odeReadError',
     +                           'ODE error reading string.')
      endif

C       .. Get input 7 : MaxStepsOde
C       Columns = 1, Rows = 1. isInt8.
      if(mxGetM(prhs(7))*mxGetN(prhs(7)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:MaxStepsOde',
     +                            'MaxStepsOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(7)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:MaxStepsOdeNonInt',
     +                           'MaxStepsOde must be an integer.')
      endif
      msPr = mxGetPr(prhs(7))
      call mxCopyPtrToReal8(msPr,msD,1)
      ms = nint(msD)

C       .. Get input 8 : fft
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(8)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:fftNonString',
     +                      'FreeFinalTime must be a string.')
      endif
      if(mxGetM(prhs(8)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:fftNonRowVector',
     +                      'FreeFinalTime must be a row vector.')
      endif
      strlen = mxGetM(prhs(8))*mxGetN(prhs(8))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:fftOverMaxBuf',
     +                      'FreeFinalTime max length sticked out.')
      endif
      fftPr = prhs(8)
      statu = mxGetString(fftPr,fft,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:fftReadError',
     +                      'FreeFinalTime error reading string.')
      endif

C       .. Get input 9 : indexpar
C       Columns = 1, Rows > 0. isNumeric.
      mipar = mxGetM(prhs(9))
      if(mxGetN(prhs(9)) .ne. 0 .and. mxGetN(prhs(9)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:mindexpar',
     +                  'indexpar must not have more than 1 column.')
      end if
      if(mxIsNumeric(prhs(9)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:indexparNonNumeric',
     +                           'indexpar must be an integer vector.')
      endif
      iparPr = mxGetPr(prhs(9))
      if (mipar.eq.0) then
         nl = 0
      else
         nl = 1
      endif

C       .. Get input 10 : Derivative
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(10)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:DerivativeNonString',
     +                           'Derivative must be a string.')
      endif
      if(mxGetM(prhs(10)) .ne . 1) then
         call mexErrMsgIdAndTxt('MATLAB:sjac:DerivativeNonRowVector',
     +                           'Derivative must be a row vector.')
      endif
      strlen = mxGetM(prhs(10))*mxGetN(prhs(10))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:DerivativeOverMaxBuf',
     +                           'Derivative max length sticked out.')
      endif
      derPr = prhs(10)
      statu = mxGetString(derPr,der,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:DerivativeReadError',
     +                           'Derivative error reading string.')
      endif

C       .. Get input 11 : MaxStepSizeOde
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(11))*mxGetN(prhs(11)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:MaxStepSizeOde',
     +                           'MaxStepSizeOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(11)) .eq. 0) then
      call mexErrMsgIdAndTxt ('MATLAB:sjac:MaxStepSizeOdeNonNumeric',
     +                      'MaxStepSizeOde must be a numeric scalar.')
      endif
      mssPr = mxGetPr(prhs(11))
      call mxCopyPtrToReal8(mssPr,mss,1)

C       .. Get input 12 : TolScaling
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(12))*mxGetN(prhs(12)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:sjac:TolScaling',
     +                           'TolScaling must be scalar.')
      end if
      if(mxIsNumeric(prhs(12)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:sjac:TolScalingNonNumeric',
     +                           'TolScaling must be a numeric scalar.')
      endif
      tscPr = mxGetPr(prhs(12))
      call mxCopyPtrToReal8(tscPr,tsc,1)

C-----------------------------------------------------------------------

C    .. call important init ..
      call initTtfNe(fft,nzi,mzi/2,ntf,ne)

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(ne, ne + nl, 0)
      sPr     = mxGetPr(plhs(1))

C    .. Create output flag ..
C      classid  = mxClassIDFromClassName('int8')
C      complexflag = 0
C      ndim     = 1
C      dims(1)  = 1
C      plhs(2) = mxCreateNumericArray(ndim,dims,classid,complexflag)
      plhs(2) = mxCreateDoubleMatrix(1, 1, 0)
      flagPr  = mxGetPr(plhs(2))

C     .. Computation ..
      call sjacmexint(nti,tiPr,mzi,nzi,ziPr,mpar,parPr,at,rt,ode,
     +                  ms,mipar,iparPr,der,ne,ntf,nl,sPr,flag,mss,tsc)

      flagD = dble(flag)
      call mxCopyReal8ToPtr(flagD, flagPr, 1)

      return
      end

C-----------------------------------------------------------------------
C     Computational routine
      subroutine sjacmexint(nti,tiPr,mzi,nzi,ziPr,mpar,parPr,at,rt,ode,
     +                  ms,mipar,iparPr,der,ne,ntf,nl,sPr,flag,mss,tsc)

      implicit none

      mwPointer tiPr,ziPr,parPr,iparPr
      mwPointer sPr
      double precision at,rt,mss,tsc
      integer ms,flag,ne,nl,ntf
      character*32 ode,der
      mwPointer nti,mzi,nzi,mpar,mipar

      double precision ti(nti),zi(mzi,nzi),par(mpar),ipar(mipar)
      double precision s(ne,ne+nl)

      call mxCopyPtrToReal8(tiPr,ti,nti)
      call mxCopyPtrToReal8(ziPr,zi,mzi*nzi)
      call mxCopyPtrToReal8(parPr,par,mpar)
      call mxCopyPtrToReal8(iparPr,ipar,mipar)

      call sjacint(nzi,ti,mzi/2,zi,mpar,par,mipar,ipar,nl,ne,ntf,s,
     +              flag,at,rt,ode,ms,der,mss,tsc)

      call mxCopyReal8ToPtr(s,sPr,ne*(ne+nl))

      return
      end


