function [ tisol, zisol, s, nfev, njev, flag] = ssolve(ti0,zi0,options,par)
%-------------------------------------------------------------------------------------------
%
%    ssolve (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Interface of the Fortran non linear solver (hybrj) to solve the optimal
%        control problem described by the Fortran subroutines.
%
%    Options used
%        Derivative, Display, FreeFinalTime, MaxStepsOde, MaxStepSizeOde, ODE,
%        TolOdeAbs, TolOdeRel, TolX, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [tisol,zisol,ssol,nfev,njev,flag] = ssolve(ti0,zi0,options,par)
%
%    Inputs
%
%        ti0     - real row vector, intial guess,
%                    ti0 = [t0 tf]                   : simple shooting
%                    ti0 = [t0 t1 .. t_{nbarc-1} tf] : multiple shooting
%        zi0     - real matrix, initial guess,
%                    zi0 = [z(t0) ... z(t_{nbarc-1})]
%        options - struct vector, hampathset options
%        par     - real row vector, par in hfun and efun par=[] if no parameters
%
%    Outputs
%
%        tisol   - real row vector, ti solution
%        zisol   - real matrix, zi solution
%        ssol    - real vector, value of sfun at tisol, zisol
%        nfev    - integer, number of evaluations of sfun
%        njev    - integer, number of evaluations of sjac
%        flag    - integer, solver output (should be 1)
%
%-------------------------------------------------------------------------------------------
if(nargout~=6)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min | nargout~=6)
error('wrong syntax: try help');
end

gfortranMatlab(1);
[ tisol, zisol, s, nfev, njev, flag] = ssolve_m(zi0,par,ti0,options.TolOdeAbs, ...
               options.TolOdeRel,options.TolX,options.ODE,options.MaxStepsOde, ...
               options.Display,options.FreeFinalTime,options.Derivative,options.MaxStepSizeOde, ...
               options.TolScaling);
gfortranMatlab(2);

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!
