#include "fintrf.h"
C======================================================================
#if 0
C
C     ssolve_m.F
C     .F file needs to be preprocessed to generate .for equivalent
C
#endif
C       Written on 2009-2012
C       by Oivier Cots - Math. Inst., Bourgogne Univ. / ENSEEIHT-IRIT
C       updated at INRIA Sophia Antipolis
C
C       [ tisol, zisol, s, nfev, njev, flag] = ssolve_m(zi0,par,ti0,
C       options.TolOdeAbs,options.TolOdeRel,options.TolX,options.ODE,
C       options.MaxSteps,options.Display,ntf,options.Derivative,
C       options.MaxStepSizeOde,options.TolScaling);
C
C       .. Global Arguments ..
C
C       NLHS            (input) INTEGER
C                       Number of outputs
C       PLHS            (input) MWPOINTER array
C                       Outputs
C       NRHS            (input) INTEGER
C                       Number of inputs
C       PRHS            (input) MWPOINTER array
C                       Inputs
C
C       .. Input Arguments ..
C
C       zi0             : double precision matrix
C       par             : double precision vector
C       ti0             : double precision vector
C       TolOdeAbs       : double precision
C       TolOdeRel       : double precision
C       TolX            : double precision
C       ODE             : char*32
C       MaxStepsOde     : integer
C       Display         : char*32
C       ntf             : integer
C       Derivative      : char*32
C       MaxStepSizeOde  : double precision
C       TolScaling      : double precision
C
C       .. Output Arguments
C
C       tisol       : double precision vector
C       zisol       : double precision vector
C       s           : double precision vector
C       nfev        : integer
C       njev        : integer
C       flag        : integer
C
C======================================================================
C   Gateway routine
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

C   Declaration
      implicit none

C   mexFunction arguments
      mwPointer plhs(*), prhs(*)
      integer nlhs, nrhs

C   Arguments for checking
      integer nlhs0, nrhs0
      parameter(nlhs0 = 6)
      parameter(nrhs0 = 13)

C   Function declarations
      mwPointer mxCreateDoubleMatrix, mxGetPr, mxGetM, mxGetN
      mwPointer mxCreateString, mxIsInt8, mxCreateNumericArray
      integer mxGetString, mxIsNumeric, mxIsChar, statu
      integer*4 mxClassIDFromClassName

C   Arguments for mxCreateNumericArray
      integer*4 classid, complexflag
      mwSize ndim, dims(2)

C   Pointers to input/output arguments
      mwPointer zi0Pr,parPr,ti0Pr,atPr,rtPr,xtPr,odePr,derPr,fftPr
      mwPointer msPr,dspPr,flagPr,nfevPr,njevPr
      mwPointer tioPr,zioPr,sPr,mssPr,tscPr
      integer nfev,njev,flag

C   Real, char or integer associated to pointer argument
      double precision at,rt,xt,msD,nfevD,njevD,flagD,mss,tsc
      character*32 ode,dsp,der,fft
      integer ms,ntf

C   Array information
      mwPointer mzi0,nzi0,mpar,nti0
      integer ne
      mwSize maxBuf
      parameter(maxBuf = 32)
      mwPointer strlen

C-----------------------------------------------------------------------
C       Check for proper number of arguments. 
      if(nrhs .ne. nrhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:nInput',
     +                           'Wrong number of inputs.')
      elseif(nlhs .ne. nlhs0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:nOutput',
     +                           'Wrong number of outputs.')
      endif

C-----------------------------------------------------------------------

C       .. Get input 1 : zi0
C       Columns = 1, Rows > 0. isNumeric.
      mzi0 = mxGetM(prhs(1))
      nzi0 = mxGetN(prhs(1))
      if(mod(mzi0,2) .ne. 0) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:mzi0',
     +                       'zi0 must have an even number of rows.')
      end if
      if(mxIsNumeric(prhs(1)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:zi0NonNumeric',
     +                           'zi0 must be a numeric array.')
      endif
      zi0Pr = mxGetPr(prhs(1))

C       .. Get input 2 : par
C       Columns = 1, Rows > 0. isNumeric.
      mpar = mxGetM(prhs(2))
      if(mxGetN(prhs(2)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:mpar',
     +                           'par must have one column.')
      end if
      if(mxIsNumeric(prhs(2)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:parNonNumeric',
     +                           'par must be a numeric vector.')
      endif
      parPr = mxGetPr(prhs(2))


C       .. Get input 3 : ti0
C       Columns > 0, Rows = 1. isNumeric.
      nti0 = mxGetN(prhs(3))
      if(mxGetM(prhs(3)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:mti0',
     +                           'ti0 must have one row.')
      end if
      if(nti0 .le. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:nti0',
     +                           'Check that ti0=[t0 t1 tm-1 tf].')
      end if
      if(nti0 .ne. nzi0+1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:nti0',
     +                     'ti0 must have one column more than zi0.')
      end if
      if(mxIsNumeric(prhs(3)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:ti0NonNumeric',
     +                           'ti0 must be a numeric array.')
      endif
      ti0Pr = mxGetPr(prhs(3))

C       .. Get input 4 : TolOdeAbs
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(4))*mxGetN(prhs(4)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolOdeAbs',
     +                           'TolOdeAbs must be scalar.')
      end if
      if(mxIsNumeric(prhs(4)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolOdeAbsNonNumeric',
     +                           'TolOdeAbs must be a numeric scalar.')
      endif
      atPr = mxGetPr(prhs(4))
      call mxCopyPtrToReal8(atPr,at,1)

C       .. Get input 5 : TolOdeRel
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(5))*mxGetN(prhs(5)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolOdeRel',
     +                           'TolOdeRel must be scalar.')
      end if
      if(mxIsNumeric(prhs(5)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolOdeRelNonNumeric',
     +                           'TolOdeRel must be a numeric scalar.')
      endif
      rtPr = mxGetPr(prhs(5))
      call mxCopyPtrToReal8(rtPr,rt,1)

C       .. Get input 6 : TolX 
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(6))*mxGetN(prhs(6)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolX',
     +                           'TolX must be scalar.')
      end if
      if(mxIsNumeric(prhs(6)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolXNonNumeric',
     +                           'TolX must be a numeric scalar.')
      endif
      xtPr = mxGetPr(prhs(6))
      call mxCopyPtrToReal8(xtPr,xt,1)

C       .. Get input 7 : ODE
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(7)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:odeNonString',
     +                           'ODE must be a string.')
      endif
      if(mxGetM(prhs(7)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:odeNonRowVector',
     +                           'ODE must be a row vector.')
      endif
      strlen = mxGetM(prhs(7))*mxGetN(prhs(7))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:odeOverMaxBuf',
     +                           'ODE max length sticked out.')
      endif
      odePr = prhs(7)
      statu = mxGetString(odePr,ode,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:odeReadError',
     +                           'ODE error reading string.')
      endif

C       .. Get input 8 : MaxStepsOde
C       Columns = 1, Rows = 1. isInt8.
      if(mxGetM(prhs(8))*mxGetN(prhs(8)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxStepsOde',
     +                            'MaxStepsOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(8)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxStepsOdeNonInt',
     +                           'MaxStepsOde must be an integer.')
      endif
      msPr = mxGetPr(prhs(8))
      call mxCopyPtrToReal8(msPr,msD,1)
      ms=nint(msD)

CC       .. Get input 8 : MaxSteps
CC       Columns = 1, Rows = 1. isInt8.
C      if(mxGetM(prhs(8))*mxGetN(prhs(8)) .ne. 1) then
C          call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxSteps',
C     +                            'MaxSteps must be scalar.')
C      end if
C      if(mxIsInt8(prhs(8)) .eq. 0) then
C         call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxStepsNonInt',
C     +                           'MaxSteps must be an integer.')
C      endif
C      msPr = mxGetPr(prhs(8))
C      call mxCopyPtrToInteger1(msPr,ms,1)


C       .. Get input 9 : Display
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(9)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DisplayNonString',
     +                           'Display must be a string.')
      endif
      if(mxGetM(prhs(9)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DisplayNonRowVector',
     +                           'Display must be a row vector.')
      endif
      strlen = mxGetM(prhs(9))*mxGetN(prhs(9))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DisplayOverMaxBuf',
     +                           'Display max length sticked out.')
      endif
      dspPr = prhs(9)
      statu = mxGetString(dspPr,dsp,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DisplayReadError',
     +                           'Display error reading string.')
      endif

C       .. Get input 10 : fft
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(10)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:fftNonString',
     +                      'FreeFinalTime must be a string.')
      endif
      if(mxGetM(prhs(10)) .ne . 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:fftNonRowVector',
     +                      'FreeFinalTime must be a row vector.')
      endif
      strlen = mxGetM(prhs(10))*mxGetN(prhs(10))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:fftOverMaxBuf',
     +                      'FreeFinalTime max length sticked out.')
      endif
      fftPr = prhs(10)
      statu = mxGetString(fftPr,fft,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:fftReadError',
     +                      'FreeFinalTime error reading string.')
      endif

C       .. Get input 11 : Derivative
C       Columns = 0, Rows <= 32. isChar.
      if(mxIsChar(prhs(11)) .ne. 1) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DerivativeNonString',
     +                           'Derivative must be a string.')
      endif
      if(mxGetM(prhs(11)) .ne . 1) then
         call mexErrMsgIdAndTxt('MATLAB:ssolve:DerivativeNonRowVector',
     +                           'Derivative must be a row vector.')
      endif
      strlen = mxGetM(prhs(11))*mxGetN(prhs(11))
      if(strlen .gt. maxBuf) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DerivativeOverMaxBuf',
     +                           'Derivative max length sticked out.')
      endif
      derPr = prhs(11)
      statu = mxGetString(derPr,der,maxbuf)
      if(statu .ne. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:DerivativeReadError',
     +                           'Derivative error reading string.')
      endif

C       .. Get input 12 : MaxStepSizeOde
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(12))*mxGetN(prhs(12)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxStepSizeOde',
     +                           'MaxStepSizeOde must be scalar.')
      end if
      if(mxIsNumeric(prhs(12)) .eq. 0) then
      call mexErrMsgIdAndTxt ('MATLAB:ssolve:MaxStepSizeOdeNonNumeric',
     +                      'MaxStepSizeOde must be a numeric scalar.')
      endif
      mssPr = mxGetPr(prhs(12))
      call mxCopyPtrToReal8(mssPr,mss,1)

C       .. Get input 13 : TolScaling
C       Columns = 1, Rows = 1. isNumeric.
      if(mxGetM(prhs(13))*mxGetN(prhs(13)) .ne. 1) then
          call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolScaling',
     +                           'TolScaling must be scalar.')
      end if
      if(mxIsNumeric(prhs(13)) .eq. 0) then
         call mexErrMsgIdAndTxt ('MATLAB:ssolve:TolScalingNonNumeric',
     +                           'TolScaling must be a numeric scalar.')
      endif
      tscPr = mxGetPr(prhs(13))
      call mxCopyPtrToReal8(tscPr,tsc,1)

C-----------------------------------------------------------------------

C    .. call important init ..
      call initTtfNe(fft,nzi0,mzi0/2,ntf,ne)

CC    .. Create output nfev, njev and flag ..
C      classid  = mxClassIDFromClassName('int8')
C      complexflag = 0
C      ndim     = 1
C      dims(1)  = 1
C
C      plhs(4) = mxCreateNumericArray(ndim,dims,classid,complexflag)
C      nfevPr  = mxGetPr(plhs(4))
C
C      plhs(5) = mxCreateNumericArray(ndim,dims,classid,complexflag)
C      njevPr  = mxGetPr(plhs(5))
C
C      plhs(6) = mxCreateNumericArray(ndim,dims,classid,complexflag)
C      flagPr  = mxGetPr(plhs(6))

      plhs(4) = mxCreateDoubleMatrix(1, 1, 0)
      nfevPr  = mxGetPr(plhs(4))

      plhs(5) = mxCreateDoubleMatrix(1, 1, 0)
      njevPr  = mxGetPr(plhs(5))

      plhs(6) = mxCreateDoubleMatrix(1, 1, 0)
      flagPr  = mxGetPr(plhs(6))

C    .. Create outputs ..
      plhs(1) = mxCreateDoubleMatrix(1, nti0, 0)
      tioPr   = mxGetPr(plhs(1))

      plhs(2) = mxCreateDoubleMatrix(mzi0, nzi0, 0)
      zioPr   = mxGetPr(plhs(2))

      plhs(3) = mxCreateDoubleMatrix(ne, 1, 0)
      sPr     = mxGetPr(plhs(3))

C     .. Computation ..
      call ssolvemexint(nti0,ti0Pr,mzi0,nzi0,zi0Pr,mpar,parPr,nfev,njev,
     +      flag,at,rt,xt,ode,ms,dsp,der,tioPr,zioPr,ne,ntf,sPr,mss,tsc)

      flagD = dble(flag)
      nfevD = dble(nfev)
      njevD = dble(njev)
      call mxCopyReal8ToPtr(flagD, flagPr, 1)
      call mxCopyReal8ToPtr(nfevD, nfevPr, 1)
      call mxCopyReal8ToPtr(njevD, njevPr, 1)

      return
      end

C-----------------------------------------------------------------------
C     Computational routine
      subroutine ssolvemexint(nti0,ti0Pr,mzi0,nzi0,zi0Pr,mpar,parPr,
     +                          nfev,njev,flag,at,rt,xt,ode,ms,dsp,der,
     *                          tioPr,zioPr,ne,ntf,sPr,mss,tsc)

      implicit none

      mwPointer zi0Pr,parPr,ti0Pr
      mwPointer tioPr,zioPr,sPr
      integer nfev,njev,flag
      double precision at,rt,xt,mss,tsc
      character*32 ode,dsp,der
      integer ms,ne,ntf
      mwPointer mzi0,nzi0,mpar,nti0

      double precision ti0(nti0),zi0(mzi0,nzi0),par(mpar)
      double precision tio(nti0),zio(mzi0,nzi0),s(ne)

      call mxCopyPtrToReal8(ti0Pr,ti0,nti0)
      call mxCopyPtrToReal8(zi0Pr,zi0,mzi0*nzi0)
      call mxCopyPtrToReal8(parPr,par,mpar)

      call ssolveint(nzi0,ti0,mzi0/2,zi0,mpar,par,tio,zio,ne,ntf,s,
     +              nfev,njev,flag,at,rt,xt,ode,ms,dsp,der,mss,tsc)


      call mxCopyReal8ToPtr(tio,tioPr,nti0)
      call mxCopyReal8ToPtr(zio,zioPr,mzi0*nzi0)
      call mxCopyReal8ToPtr(s  ,sPr  ,ne)

      return
      end

