!!    @brief True Hamiltonian.
!!    @param[in] t      Time
!!    @param[in] n      State dimension
!!    @param[in] z      State and adjoint state at t
!!    @param[in] nbarc  Number of arcs : nbarc = 1 for simple shooting
!!    @param[in] iarc   Index of the current arc
!!    @param[in] lpar   Number of optional parameters
!!    @param[in] par    Optional parameters
!!
!!    @param[out] H   True Hamiltonian
!!    @attention      The vector par can be used for
!!                    constant values or homotopic parameters.
        Subroutine hfun(t,n,z,nbarc,iarc,lpar,par,H)
        implicit none

        integer, intent(in)                             :: n,lpar,iarc,nbarc
        double precision, intent(in)                    :: t
        double precision, dimension(2*n),  intent(in)   :: z
        double precision, dimension(lpar), intent(in)   :: par
        double precision, intent(out)                   :: H

        !Local declarations
        double precision :: x,v,px,pv,alpha,lambda,u

        IF (nbarc.NE.1) THEN
            call printandstop('Error: wrong number of arcs.')
        END IF
        IF (n.NE.2) THEN
            call printandstop('Error: wrong state dimension.')
        END IF
        IF (lpar.NE.6) THEN
            call printandstop('Error: wrong par dimension.')
        END IF

        !dot(x) = v
        !dox(v) = -alpha v^2 + lambda u
        !min u^2
        lambda  = par(5)
        alpha   = par(6)
        x       = z(1)
        v       = z(2)
        px      = z(n+1)
        pv      = z(n+2)

        u       = lambda*pv/2
        H       = -u**2 + px * v + pv * (-alpha * v**2 + lambda * u)

        end subroutine hfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

