!!********************************************************************
!>  \ingroup problemDefinition
!!  \brief True Hamiltonian.
!!  \param[in] t      Time
!!  \param[in] n      State dimension
!!  \param[in] z      State and adjoint state at t
!!  \param[in] nbarc  Number of arcs : nbarc = 1 for simple shooting
!!  \param[in] iarc   Index of the current arc
!!  \param[in] lpar   Number of optional parameters
!!  \param[in] par    Optional parameters
!!
!!  \param[out] H   True Hamiltonian
!!  \attention      The vector par can be used for
!!                    constant values or homotopic parameters.
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
        Subroutine hfun(t,n,z,nbarc,iarc,lpar,par,H)
            implicit none
            integer, intent(in)                             :: n,lpar,iarc,nbarc
            double precision, intent(in)                    :: t
            double precision, dimension(2*n),  intent(in)   :: z
            double precision, dimension(lpar), intent(in)   :: par
            double precision, intent(out)                   :: H

            IF (nbarc.NE.0) THEN
                call printandstop('Error: wrong number of arcs.')
            END IF
            IF (n.NE.0) THEN
                call printandstop('Error: wrong state dimension.')
            END IF
            IF (lpar.NE.0) THEN
                call printandstop('Error: wrong par dimension.')
            END IF

            H = 0d0

        end subroutine hfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

